# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`ui` module provides standard UI components for OpenLP.
"""
import logging

from PyQt4 import QtCore, QtGui

from openlp.core.lib import build_icon, Receiver, translate
from openlp.core.utils.actions import ActionList

log = logging.getLogger(__name__)

class UiStrings(object):
    """
    Provide standard strings for objects to use.
    """
    __instance__ = None

    def __new__(cls):
        """
        Override the default object creation method to return a single instance.
        """
        if not cls.__instance__:
            cls.__instance__ = object.__new__(cls)
        return cls.__instance__

    def __init__(self):
        """
        These strings should need a good reason to be retranslated elsewhere.
        Should some/more/less of these have an &amp; attached?
        """
        self.About = translate('OpenLP.Ui', 'About')
        self.Add = translate('OpenLP.Ui', '&Add')
        self.Advanced = translate('OpenLP.Ui', 'Advanced')
        self.AllFiles = translate('OpenLP.Ui', 'All Files')
        self.Bottom = translate('OpenLP.Ui', 'Bottom')
        self.Browse = translate('OpenLP.Ui', 'Browse...')
        self.Cancel = translate('OpenLP.Ui', 'Cancel')
        self.CCLINumberLabel = translate('OpenLP.Ui', 'CCLI number:')
        self.CreateService = translate('OpenLP.Ui', 'Create a new service.')
        self.ConfirmDelete = translate('OpenLP.Ui', 'Confirm Delete')
        self.Continuous = translate('OpenLP.Ui', 'Continuous')
        self.Default = unicode(translate('OpenLP.Ui', 'Default'))
        self.Delete = translate('OpenLP.Ui', '&Delete')
        self.DisplayStyle = translate('OpenLP.Ui', 'Display style:')
        self.Duplicate = translate('OpenLP.Ui', 'Duplicate Error')
        self.Edit = translate('OpenLP.Ui', '&Edit')
        self.EmptyField = translate('OpenLP.Ui', 'Empty Field')
        self.Error = translate('OpenLP.Ui', 'Error')
        self.Export = translate('OpenLP.Ui', 'Export')
        self.File = translate('OpenLP.Ui', 'File')
        self.FileNotFound = unicode(translate('OpenLP.Ui',
            'File Not Found'))
        self.FileNotFoundMessage = unicode(translate('OpenLP.Ui',
            'File %s not found.\nPlease try selecting it individually.'))
        self.FontSizePtUnit = translate('OpenLP.Ui', 'pt',
            'Abbreviated font pointsize unit')
        self.Help = translate('OpenLP.Ui', 'Help')
        self.Hours = translate('OpenLP.Ui', 'h',
            'The abbreviated unit for hours')
        self.IFdSs = translate('OpenLP.Ui', 'Invalid Folder Selected',
            'Singular')
        self.IFSs = translate('OpenLP.Ui', 'Invalid File Selected', 'Singular')
        self.IFSp = translate('OpenLP.Ui', 'Invalid Files Selected', 'Plural')
        self.Image = translate('OpenLP.Ui', 'Image')
        self.Import = translate('OpenLP.Ui', 'Import')
        self.LayoutStyle = translate('OpenLP.Ui', 'Layout style:')
        self.Live = translate('OpenLP.Ui', 'Live')
        self.LiveBGError = translate('OpenLP.Ui', 'Live Background Error')
        self.LiveToolbar = translate('OpenLP.Ui', 'Live Toolbar')
        self.Load = translate('OpenLP.Ui', 'Load')
        self.Minutes = translate('OpenLP.Ui', 'm',
            'The abbreviated unit for minutes')
        self.Middle = translate('OpenLP.Ui', 'Middle')
        self.New = translate('OpenLP.Ui', 'New')
        self.NewService = translate('OpenLP.Ui', 'New Service')
        self.NewTheme = translate('OpenLP.Ui', 'New Theme')
        self.NextTrack = translate('OpenLP.Ui', 'Next Track')
        self.NFdSs = translate('OpenLP.Ui', 'No Folder Selected', 'Singular')
        self.NFSs = translate('OpenLP.Ui', 'No File Selected', 'Singular')
        self.NFSp = translate('OpenLP.Ui', 'No Files Selected', 'Plural')
        self.NISs = translate('OpenLP.Ui', 'No Item Selected', 'Singular')
        self.NISp = translate('OpenLP.Ui', 'No Items Selected', 'Plural')
        self.OLPV1 = translate('OpenLP.Ui', 'openlp.org 1.x')
        self.OLPV2 = translate('OpenLP.Ui', 'OpenLP 2.0')
        self.OpenLPStart = translate('OpenLP.Ui', 'OpenLP is already running. '
            'Do you wish to continue?')
        self.OpenService = translate('OpenLP.Ui', 'Open service.')
        self.PlaySlidesInLoop = translate('OpenLP.Ui','Play Slides in Loop')
        self.PlaySlidesToEnd = translate('OpenLP.Ui','Play Slides to End')
        self.Preview = translate('OpenLP.Ui', 'Preview')
        self.PrintService = translate('OpenLP.Ui', 'Print Service')
        self.ReplaceBG = translate('OpenLP.Ui', 'Replace Background')
        self.ReplaceLiveBG = translate('OpenLP.Ui', 'Replace live background.')
        self.ResetBG = translate('OpenLP.Ui', 'Reset Background')
        self.ResetLiveBG = translate('OpenLP.Ui', 'Reset live background.')
        self.Seconds = translate('OpenLP.Ui', 's',
            'The abbreviated unit for seconds')
        self.SaveAndPreview = translate('OpenLP.Ui', 'Save && Preview')
        self.Search = translate('OpenLP.Ui', 'Search')
        self.SearchThemes = translate(
            'OpenLP.Ui', 'Search Themes...', 'Search bar place holder text ')
        self.SelectDelete = translate('OpenLP.Ui', 'You must select an item '
            'to delete.')
        self.SelectEdit = translate('OpenLP.Ui', 'You must select an item to '
            'edit.')
        self.Settings = translate('OpenLP.Ui', 'Settings')
        self.SaveService = translate('OpenLP.Ui', 'Save Service')
        self.Service = translate('OpenLP.Ui', 'Service')
        self.Split = translate('OpenLP.Ui', 'Optional &Split')
        self.SplitToolTip = translate('OpenLP.Ui', 'Split a slide into two '
            'only if it does not fit on the screen as one slide.')
        self.StartTimeCode = unicode(translate('OpenLP.Ui', 'Start %s'))
        self.StopPlaySlidesInLoop = translate('OpenLP.Ui',
            'Stop Play Slides in Loop')
        self.StopPlaySlidesToEnd = translate('OpenLP.Ui',
            'Stop Play Slides to End')
        self.Theme = translate('OpenLP.Ui', 'Theme', 'Singular')
        self.Themes = translate('OpenLP.Ui', 'Themes', 'Plural')
        self.Tools = translate('OpenLP.Ui', 'Tools')
        self.Top = translate('OpenLP.Ui', 'Top')
        self.UnsupportedFile = translate('OpenLP.Ui', 'Unsupported File')
        self.VersePerSlide = translate('OpenLP.Ui', 'Verse Per Slide')
        self.VersePerLine = translate('OpenLP.Ui', 'Verse Per Line')
        self.Version = translate('OpenLP.Ui', 'Version')
        self.View = translate('OpenLP.Ui', 'View')
        self.ViewMode = translate('OpenLP.Ui', 'View Mode')


def add_welcome_page(parent, image):
    """
    Generate an opening welcome page for a wizard using a provided image.

    ``parent``
        A ``QWizard`` object to add the welcome page to.

    ``image``
        A splash image for the wizard.
    """
    parent.welcomePage = QtGui.QWizardPage()
    parent.welcomePage.setPixmap(QtGui.QWizard.WatermarkPixmap,
        QtGui.QPixmap(image))
    parent.welcomePage.setObjectName(u'WelcomePage')
    parent.welcomeLayout = QtGui.QVBoxLayout(parent.welcomePage)
    parent.welcomeLayout.setObjectName(u'WelcomeLayout')
    parent.titleLabel = QtGui.QLabel(parent.welcomePage)
    parent.titleLabel.setObjectName(u'TitleLabel')
    parent.welcomeLayout.addWidget(parent.titleLabel)
    parent.welcomeLayout.addSpacing(40)
    parent.informationLabel = QtGui.QLabel(parent.welcomePage)
    parent.informationLabel.setWordWrap(True)
    parent.informationLabel.setObjectName(u'InformationLabel')
    parent.welcomeLayout.addWidget(parent.informationLabel)
    parent.welcomeLayout.addStretch()
    parent.addPage(parent.welcomePage)


def create_button_box(dialog, name, standard_buttons, custom_buttons=[]):
    """
    Creates a QDialogButtonBox with the given buttons. The ``accepted()`` and
    ``rejected()`` signals of the button box are connected with the dialogs
    ``accept()`` and ``reject()`` slots.

    ``dialog``
        The parent object. This has to be a ``QDialog`` descendant.

    ``name``
        A string which is set as object name.

    ``standard_buttons``
        A list of strings for the used buttons. It might contain: ``ok``,
        ``save``, ``cancel``, ``close``, and ``defaults``.

    ``custom_buttons``
        A list of additional buttons. If a item is a instance of
        QtGui.QAbstractButton it is added with QDialogButtonBox.ActionRole.
        Otherwhise the item has to be a tuple of a button and a ButtonRole.
    """
    buttons = QtGui.QDialogButtonBox.NoButton
    if u'ok' in standard_buttons:
        buttons |= QtGui.QDialogButtonBox.Ok
    if u'save' in standard_buttons:
        buttons |= QtGui.QDialogButtonBox.Save
    if u'cancel' in standard_buttons:
        buttons |= QtGui.QDialogButtonBox.Cancel
    if u'close' in standard_buttons:
        buttons |= QtGui.QDialogButtonBox.Close
    if u'defaults' in standard_buttons:
        buttons |= QtGui.QDialogButtonBox.RestoreDefaults
    button_box = QtGui.QDialogButtonBox(dialog)
    button_box.setObjectName(name)
    button_box.setStandardButtons(buttons)
    for button in custom_buttons:
        if isinstance(button, QtGui.QAbstractButton):
            button_box.addButton(button, QtGui.QDialogButtonBox.ActionRole)
        else:
            button_box.addButton(*button)
    QtCore.QObject.connect(button_box, QtCore.SIGNAL(u'accepted()'),
        dialog.accept)
    QtCore.QObject.connect(button_box, QtCore.SIGNAL(u'rejected()'),
        dialog.reject)
    return button_box


def critical_error_message_box(title=None, message=None, parent=None,
    question=False):
    """
    Provides a standard critical message box for errors that OpenLP displays
    to users.

    ``title``
        The title for the message box.

    ``message``
        The message to display to the user.

    ``parent``
        The parent UI element to attach the dialog to.

    ``question``
        Should this message box question the user.
    """
    if question:
        return QtGui.QMessageBox.critical(parent, UiStrings().Error, message,
            QtGui.QMessageBox.StandardButtons(
            QtGui.QMessageBox.Yes | QtGui.QMessageBox.No))
    data = {u'message': message}
    data[u'title'] = title if title else UiStrings().Error
    return Receiver.send_message(u'openlp_error_message', data)


def create_horizontal_adjusting_combo_box(parent, name):
    """
    Creates a QComboBox with adapting width for media items.

    ``parent``
        The parent widget.

    ``name``
        A string set as object name for the combo box.
    """
    combo = QtGui.QComboBox(parent)
    combo.setObjectName(name)
    combo.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToMinimumContentsLength)
    combo.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
    return combo


def create_button(parent, name, **kwargs):
    """
    Return an button with the object name set and the given parameters.

    ``parent``
        A QtCore.QWidget for the buttons parent (required).

    ``name``
        A string which is set as object name (required).

    ``role``
        A string which can have one value out of ``delete``, ``up``, and
        ``down``. This decides about default values for properties like text,
        icon, or tooltip.

    ``text``
        A string for the action text.

    ``icon``
        Either a QIcon, a resource string, or a file location string for the
        action icon.

    ``tooltip``
        A string for the action tool tip.

    ``enabled``
        False in case the button should be disabled.
    """
    if u'role' in kwargs:
        role = kwargs.pop(u'role')
        if role == u'delete':
            kwargs.setdefault(u'text', UiStrings().Delete)
            kwargs.setdefault(u'tooltip',
                translate('OpenLP.Ui', 'Delete the selected item.'))
        elif role == u'up':
            kwargs.setdefault(u'icon', u':/services/service_up.png')
            kwargs.setdefault(u'tooltip',
                translate('OpenLP.Ui', 'Move selection up one position.'))
        elif role == u'down':
            kwargs.setdefault(u'icon', u':/services/service_down.png')
            kwargs.setdefault(u'tooltip',
                translate('OpenLP.Ui', 'Move selection down one position.'))
        else:
            log.warn(u'The role "%s" is not defined in create_push_button().',
                role)
    if kwargs.pop(u'class', u'') == u'toolbutton':
        button = QtGui.QToolButton(parent)
    else:
        button = QtGui.QPushButton(parent)
    button.setObjectName(name)
    if kwargs.get(u'text'):
        button.setText(kwargs.pop(u'text'))
    if kwargs.get(u'icon'):
        button.setIcon(build_icon(kwargs.pop(u'icon')))
    if kwargs.get(u'tooltip'):
        button.setToolTip(kwargs.pop(u'tooltip'))
    if not kwargs.pop(u'enabled', True):
        button.setEnabled(False)
    if kwargs.get(u'click'):
        QtCore.QObject.connect(button, QtCore.SIGNAL(u'clicked()'),
            kwargs.pop(u'click'))
    for key in kwargs.keys():
        if key not in [u'text', u'icon', u'tooltip', u'click']:
            log.warn(u'Parameter %s was not consumed in create_button().', key)
    return button


def create_action(parent, name, **kwargs):
    """
    Return an action with the object name set and the given parameters.

    ``parent``
        A QtCore.QObject for the actions parent (required).

    ``name``
        A string which is set as object name (required).

    ``text``
        A string for the action text.

    ``icon``
        Either a QIcon, a resource string, or a file location string for the
        action icon.

    ``tooltip``
        A string for the action tool tip.

    ``statustip``
        A string for the action status tip.

    ``checked``
        A bool for the state. If ``None`` the Action is not checkable.

    ``enabled``
        False in case the action should be disabled.

    ``visible``
        False in case the action should be hidden.

    ``separator``
        True in case the action will be considered a separator.

    ``data``
        Data which is set as QVariant type.

    ``shortcuts``
        A QList<QKeySequence> (or a list of strings) which are set as shortcuts.

    ``context``
        A context for the shortcut execution.

    ``category``
        A category the action should be listed in the shortcut dialog.

    ``triggers``
        A slot which is connected to the actions ``triggered()`` slot.
    """
    action = QtGui.QAction(parent)
    action.setObjectName(name)
    if kwargs.get(u'text'):
        action.setText(kwargs.pop(u'text'))
    if kwargs.get(u'icon'):
        action.setIcon(build_icon(kwargs.pop(u'icon')))
    if kwargs.get(u'tooltip'):
        action.setToolTip(kwargs.pop(u'tooltip'))
    if kwargs.get(u'statustip'):
        action.setStatusTip(kwargs.pop(u'statustip'))
    if kwargs.get(u'checked') is not None:
        action.setCheckable(True)
        action.setChecked(kwargs.pop(u'checked'))
    if not kwargs.pop(u'enabled', True):
        action.setEnabled(False)
    if not kwargs.pop(u'visible', True):
        action.setVisible(False)
    if kwargs.pop(u'separator', False):
        action.setSeparator(True)
    if u'data' in kwargs:
        action.setData(QtCore.QVariant(kwargs.pop(u'data')))
    if kwargs.get(u'shortcuts'):
        action.setShortcuts(kwargs.pop(u'shortcuts'))
    if u'context' in kwargs:
        action.setShortcutContext(kwargs.pop(u'context'))
    if kwargs.get(u'category'):
        action_list = ActionList.get_instance()
        action_list.add_action(action, unicode(kwargs.pop(u'category')))
    if kwargs.get(u'triggers'):
        QtCore.QObject.connect(action, QtCore.SIGNAL(u'triggered(bool)'),
            kwargs.pop(u'triggers'))
    for key in kwargs.keys():
        if key not in [u'text', u'icon', u'tooltip', u'statustip', u'checked',
            u'shortcuts', u'category', u'triggers']:
            log.warn(u'Parameter %s was not consumed in create_action().', key)
    return action


def create_widget_action(parent, name=u'', **kwargs):
    """
    Return a new QAction by calling ``create_action(parent, name, **kwargs)``.
    The shortcut context defaults to ``QtCore.Qt.WidgetShortcut`` and the action
    is added to the parents action list.
    """
    kwargs.setdefault(u'context', QtCore.Qt.WidgetShortcut)
    action = create_action(parent, name, **kwargs)
    parent.addAction(action)
    return action


def set_case_insensitive_completer(cache, widget):
    """
    Sets a case insensitive text completer for a widget.

    ``cache``
        The list of items to use as suggestions.

    ``widget``
        A widget to set the completer (QComboBox or QTextEdit instance)
    """
    completer = QtGui.QCompleter(cache)
    completer.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
    widget.setCompleter(completer)


def create_valign_selection_widgets(parent):
    """
    Creates a standard label and combo box for asking users to select a
    vertical alignment.

    ``parent``
        The parent object. This should be a ``QWidget`` descendant.

    Returns a tuple of QLabel and QComboBox.
    """
    label = QtGui.QLabel(parent)
    label.setText(translate('OpenLP.Ui', '&Vertical Align:'))
    combo_box = QtGui.QComboBox(parent)
    combo_box.addItem(UiStrings().Top)
    combo_box.addItem(UiStrings().Middle)
    combo_box.addItem(UiStrings().Bottom)
    label.setBuddy(combo_box)
    return label, combo_box


def find_and_set_in_combo_box(combo_box, value_to_find):
    """
    Find a string in a combo box and set it as the selected item if present

    ``combo_box``
        The combo box to check for selected items

    ``value_to_find``
        The value to find
    """
    index = combo_box.findText(value_to_find,
        QtCore.Qt.MatchExactly)
    if index == -1:
        # Not Found.
        index = 0
    combo_box.setCurrentIndex(index)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import build_icon, translate
from openlp.core.lib.ui import create_button_box
from openlp.plugins.bibles.lib import LanguageSelection, BibleStrings
from openlp.plugins.bibles.lib.db import BiblesResourcesDB


class Ui_EditBibleDialog(object):
    def setupUi(self, editBibleDialog):
        editBibleDialog.setObjectName(u'editBibleDialog')
        editBibleDialog.resize(520, 400)
        editBibleDialog.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        editBibleDialog.setModal(True)
        self.dialogLayout = QtGui.QVBoxLayout(editBibleDialog)
        self.dialogLayout.setSpacing(8)
        self.dialogLayout.setContentsMargins(8, 8, 8, 8)
        self.dialogLayout.setObjectName(u'dialogLayout')
        self.bibleTabWidget = QtGui.QTabWidget(editBibleDialog)
        self.bibleTabWidget.setObjectName(u'BibleTabWidget')
        # Meta tab
        self.metaTab = QtGui.QWidget()
        self.metaTab.setObjectName(u'metaTab')
        self.metaTabLayout = QtGui.QVBoxLayout(self.metaTab)
        self.metaTabLayout.setObjectName(u'metaTabLayout')
        self.licenseDetailsGroupBox = QtGui.QGroupBox(self.metaTab)
        self.licenseDetailsGroupBox.setObjectName(u'licenseDetailsGroupBox')
        self.licenseDetailsLayout = QtGui.QFormLayout(
            self.licenseDetailsGroupBox)
        self.licenseDetailsLayout.setObjectName(u'licenseDetailsLayout')
        self.versionNameLabel = QtGui.QLabel(self.licenseDetailsGroupBox)
        self.versionNameLabel.setObjectName(u'versionNameLabel')
        self.versionNameEdit = QtGui.QLineEdit(self.licenseDetailsGroupBox)
        self.versionNameEdit.setObjectName(u'versionNameEdit')
        self.versionNameLabel.setBuddy(self.versionNameEdit)
        self.licenseDetailsLayout.addRow(self.versionNameLabel,
            self.versionNameEdit)
        self.copyrightLabel = QtGui.QLabel(self.licenseDetailsGroupBox)
        self.copyrightLabel.setObjectName(u'copyrightLabel')
        self.copyrightEdit = QtGui.QLineEdit(self.licenseDetailsGroupBox)
        self.copyrightEdit.setObjectName(u'copyrightEdit')
        self.copyrightLabel.setBuddy(self.copyrightEdit)
        self.licenseDetailsLayout.addRow(self.copyrightLabel,
            self.copyrightEdit)
        self.permissionsLabel = QtGui.QLabel(self.licenseDetailsGroupBox)
        self.permissionsLabel.setObjectName(u'permissionsLabel')
        self.permissionsEdit = QtGui.QLineEdit(self.licenseDetailsGroupBox)
        self.permissionsEdit.setObjectName(u'permissionsEdit')
        self.permissionsLabel.setBuddy(self.permissionsEdit)
        self.licenseDetailsLayout.addRow(self.permissionsLabel,
            self.permissionsEdit)
        self.metaTabLayout.addWidget(self.licenseDetailsGroupBox)
        self.languageSelectionGroupBox = QtGui.QGroupBox(self.metaTab)
        self.languageSelectionGroupBox.setObjectName(
            u'languageSelectionGroupBox')
        self.languageSelectionLayout = QtGui.QVBoxLayout(
            self.languageSelectionGroupBox)
        self.languageSelectionLabel = QtGui.QLabel(
            self.languageSelectionGroupBox)
        self.languageSelectionLabel.setObjectName(u'languageSelectionLabel')
        self.languageSelectionComboBox = QtGui.QComboBox(
            self.languageSelectionGroupBox)
        self.languageSelectionComboBox.setObjectName(
            u'languageSelectionComboBox')
        self.languageSelectionComboBox.addItems([u'', u'', u'', u''])
        self.languageSelectionLayout.addWidget(self.languageSelectionLabel)
        self.languageSelectionLayout.addWidget(self.languageSelectionComboBox)
        self.metaTabLayout.addWidget(self.languageSelectionGroupBox)
        self.metaTabLayout.addStretch()
        self.bibleTabWidget.addTab(self.metaTab, u'')
        # Book name tab
        self.bookNameTab = QtGui.QWidget()
        self.bookNameTab.setObjectName(u'bookNameTab')
        self.bookNameTabLayout = QtGui.QVBoxLayout(self.bookNameTab)
        self.bookNameTabLayout.setObjectName(u'bookNameTabLayout')
        self.bookNameNotice = QtGui.QLabel(self.bookNameTab)
        self.bookNameNotice.setObjectName(u'bookNameNotice')
        self.bookNameNotice.setWordWrap(True)
        self.bookNameTabLayout.addWidget(self.bookNameNotice)
        self.scrollArea = QtGui.QScrollArea(self.bookNameTab)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName(u'scrollArea')
        self.scrollArea.setHorizontalScrollBarPolicy(
            QtCore.Qt.ScrollBarAlwaysOff)
        self.bookNameWidget = QtGui.QWidget(self.scrollArea)
        self.bookNameWidget.setObjectName(u'bookNameWidget')
        self.bookNameWidgetLayout = QtGui.QFormLayout(self.bookNameWidget)
        self.bookNameWidgetLayout.setObjectName(u'bookNameWidgetLayout')
        self.bookNameLabel = {}
        self.bookNameEdit= {}
        for book in BiblesResourcesDB.get_books():
            self.bookNameLabel[book[u'abbreviation']] = QtGui.QLabel(
                self.bookNameWidget)
            self.bookNameLabel[book[u'abbreviation']].setObjectName(
                u'bookNameLabel[%s]' % book[u'abbreviation'])
            self.bookNameEdit[book[u'abbreviation']] = QtGui.QLineEdit(
                self.bookNameWidget)
            self.bookNameEdit[book[u'abbreviation']].setObjectName(
                u'bookNameEdit[%s]' % book[u'abbreviation'])
            self.bookNameWidgetLayout.addRow(
                self.bookNameLabel[book[u'abbreviation']],
                self.bookNameEdit[book[u'abbreviation']])
        self.scrollArea.setWidget(self.bookNameWidget)
        self.bookNameTabLayout.addWidget(self.scrollArea)
        self.bookNameTabLayout.addStretch()
        self.bibleTabWidget.addTab(self.bookNameTab, u'')
        # Last few bits
        self.dialogLayout.addWidget(self.bibleTabWidget)
        self.buttonBox = create_button_box(editBibleDialog, u'buttonBox',
            [u'cancel', u'save'])
        self.dialogLayout.addWidget(self.buttonBox)
        self.retranslateUi(editBibleDialog)
        QtCore.QMetaObject.connectSlotsByName(editBibleDialog)

    def retranslateUi(self, editBibleDialog):
        self.book_names = BibleStrings().BookNames
        editBibleDialog.setWindowTitle(
            translate('BiblesPlugin.EditBibleForm', 'Bible Editor'))
        # Meta tab
        self.bibleTabWidget.setTabText(
            self.bibleTabWidget.indexOf(self.metaTab),
            translate('SongsPlugin.EditBibleForm', 'Meta Data'))
        self.licenseDetailsGroupBox.setTitle(
            translate('BiblesPlugin.EditBibleForm', 'License Details'))
        self.versionNameLabel.setText(
            translate('BiblesPlugin.EditBibleForm', 'Version name:'))
        self.copyrightLabel.setText(
            translate('BiblesPlugin.EditBibleForm', 'Copyright:'))
        self.permissionsLabel.setText(
            translate('BiblesPlugin.EditBibleForm', 'Permissions:'))
        self.languageSelectionGroupBox.setTitle(translate(
            'BiblesPlugin.EditBibleForm', 'Default Bible Language'))
        self.languageSelectionLabel.setText(
            translate('BiblesPlugin.EditBibleForm',
            'Book name language in search field, search results and on '
            'display:'))
        self.languageSelectionComboBox.setItemText(0,
            translate('BiblesPlugin.EditBibleForm', 'Global Settings'))
        self.languageSelectionComboBox.setItemText(LanguageSelection.Bible + 1,
            translate('BiblesPlugin.EditBibleForm', 'Bible Language'))
        self.languageSelectionComboBox.setItemText(
            LanguageSelection.Application + 1,
            translate('BiblesPlugin.EditBibleForm', 'Application Language'))
        self.languageSelectionComboBox.setItemText(
            LanguageSelection.English + 1,
            translate('BiblesPlugin.EditBibleForm', 'English'))
        # Book name tab
        self.bibleTabWidget.setTabText(
            self.bibleTabWidget.indexOf(self.bookNameTab),
            translate('SongsPlugin.EditBibleForm', 'Custom Book Names'))
        for book in BiblesResourcesDB.get_books():
            self.bookNameLabel[book[u'abbreviation']].setText(
                u'%s:' % unicode(self.book_names[book[u'abbreviation']]))



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
The :mod:`advancedtab` provides an advanced settings facility.
"""
from datetime import datetime, timedelta
import logging
import os
import sys

from PyQt4 import QtCore, QtGui

from openlp.core.lib import SettingsTab, translate, build_icon,  Receiver
from openlp.core.lib.settings import Settings
from openlp.core.lib.ui import UiStrings
from openlp.core.utils import get_images_filter, AppLocation, format_time
from openlp.core.lib import SlideLimits

log = logging.getLogger(__name__)

class AdvancedTab(SettingsTab):
    """
    The :class:`AdvancedTab` manages the advanced settings tab including the UI
    and the loading and saving of the displayed settings.
    """
    def __init__(self, parent):
        """
        Initialise the settings tab
        """
        self.displayChanged = False
        # 7 stands for now, 0 to 6 is Monday to Sunday.
        self.defaultServiceDay = 7
        # 11 o'clock is the most popular time for morning service.
        self.defaultServiceHour = 11
        self.defaultServiceMinute = 0
        self.defaultServiceName = unicode(translate('OpenLP.AdvancedTab',
            'Service %Y-%m-%d %H-%M',
            'This may not contain any of the following characters: '
            '/\\?*|<>\[\]":+\n'
            'See http://docs.python.org/library/datetime.html'
            '#strftime-strptime-behavior for more information.'))
        self.defaultImage = u':/graphics/openlp-splash-screen.png'
        self.defaultColor = u'#ffffff'
        self.dataExists = False
        self.iconPath = u':/system/system_settings.png'
        advanced_translated = translate('OpenLP.AdvancedTab', 'Advanced')
        SettingsTab.__init__(self, parent, u'Advanced', advanced_translated)

    def setupUi(self):
        """
        Configure the UI elements for the tab.
        """
        self.setObjectName(u'AdvancedTab')
        SettingsTab.setupUi(self)
        self.uiGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.uiGroupBox.setObjectName(u'uiGroupBox')
        self.uiLayout = QtGui.QFormLayout(self.uiGroupBox)
        self.uiLayout.setObjectName(u'uiLayout')
        self.recentLabel = QtGui.QLabel(self.uiGroupBox)
        self.recentLabel.setObjectName(u'recentLabel')
        self.recentSpinBox = QtGui.QSpinBox(self.uiGroupBox)
        self.recentSpinBox.setObjectName(u'recentSpinBox')
        self.recentSpinBox.setMinimum(0)
        self.uiLayout.addRow(self.recentLabel, self.recentSpinBox)
        self.mediaPluginCheckBox = QtGui.QCheckBox(self.uiGroupBox)
        self.mediaPluginCheckBox.setObjectName(u'mediaPluginCheckBox')
        self.uiLayout.addRow(self.mediaPluginCheckBox)
        self.doubleClickLiveCheckBox = QtGui.QCheckBox(self.uiGroupBox)
        self.doubleClickLiveCheckBox.setObjectName(u'doubleClickLiveCheckBox')
        self.uiLayout.addRow(self.doubleClickLiveCheckBox)
        self.singleClickPreviewCheckBox = QtGui.QCheckBox(self.uiGroupBox)
        self.singleClickPreviewCheckBox.setObjectName(
            u'singleClickPreviewCheckBox')
        self.uiLayout.addRow(self.singleClickPreviewCheckBox)
        self.expandServiceItemCheckBox = QtGui.QCheckBox(self.uiGroupBox)
        self.expandServiceItemCheckBox.setObjectName(
            u'expandServiceItemCheckBox')
        self.uiLayout.addRow(self.expandServiceItemCheckBox)
        self.enableAutoCloseCheckBox = QtGui.QCheckBox(self.uiGroupBox)
        self.enableAutoCloseCheckBox.setObjectName(
            u'enableAutoCloseCheckBox')
        self.uiLayout.addRow(self.enableAutoCloseCheckBox)
        self.leftLayout.addWidget(self.uiGroupBox)
        # Default service name
        self.serviceNameGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.serviceNameGroupBox.setObjectName(u'serviceNameGroupBox')
        self.serviceNameLayout = QtGui.QFormLayout(
            self.serviceNameGroupBox)
        self.serviceNameCheckBox = QtGui.QCheckBox(
            self.serviceNameGroupBox)
        self.serviceNameCheckBox.setObjectName(u'serviceNameCheckBox')
        self.serviceNameLayout.setObjectName(u'serviceNameLayout')
        self.serviceNameLayout.addRow(self.serviceNameCheckBox)
        self.serviceNameTimeLabel = QtGui.QLabel(self.serviceNameGroupBox)
        self.serviceNameTimeLabel.setObjectName(u'serviceNameTimeLabel')
        self.serviceNameDay = QtGui.QComboBox(
            self.serviceNameGroupBox)
        self.serviceNameDay.addItems(
            [u'', u'', u'', u'', u'', u'', u'', u''])
        self.serviceNameDay.setObjectName(
            u'serviceNameDay')
        self.serviceNameTime = QtGui.QTimeEdit(self.serviceNameGroupBox)
        self.serviceNameTime.setObjectName(u'serviceNameTime')
        self.serviceNameTimeHBox = QtGui.QHBoxLayout()
        self.serviceNameTimeHBox.setObjectName(u'serviceNameTimeHBox')
        self.serviceNameTimeHBox.addWidget(self.serviceNameDay)
        self.serviceNameTimeHBox.addWidget(self.serviceNameTime)
        self.serviceNameLayout.addRow(self.serviceNameTimeLabel,
            self.serviceNameTimeHBox)
        self.serviceNameLabel = QtGui.QLabel(self.serviceNameGroupBox)
        self.serviceNameLabel.setObjectName(u'serviceNameLabel')
        self.serviceNameEdit = QtGui.QLineEdit(self.serviceNameGroupBox)
        self.serviceNameEdit.setObjectName(u'serviceNameEdit')
        self.serviceNameEdit.setValidator(QtGui.QRegExpValidator(
            QtCore.QRegExp(r'[^/\\?*|<>\[\]":+]+'), self))
        self.serviceNameRevertButton = QtGui.QToolButton(
            self.serviceNameGroupBox)
        self.serviceNameRevertButton.setObjectName(
            u'serviceNameRevertButton')
        self.serviceNameRevertButton.setIcon(
            build_icon(u':/general/general_revert.png'))
        self.serviceNameHBox = QtGui.QHBoxLayout()
        self.serviceNameHBox.setObjectName(u'serviceNameHBox')
        self.serviceNameHBox.addWidget(self.serviceNameEdit)
        self.serviceNameHBox.addWidget(self.serviceNameRevertButton)
        self.serviceNameLayout.addRow(self.serviceNameLabel,
            self.serviceNameHBox)
        self.serviceNameExampleLabel = QtGui.QLabel(
            self.serviceNameGroupBox)
        self.serviceNameExampleLabel.setObjectName(
            u'serviceNameExampleLabel')
        self.serviceNameExample = QtGui.QLabel(self.serviceNameGroupBox)
        self.serviceNameExample.setObjectName(u'serviceNameExample')
        self.serviceNameLayout.addRow(self.serviceNameExampleLabel,
            self.serviceNameExample)
        self.leftLayout.addWidget(self.serviceNameGroupBox)
        # Data Directory
        self.dataDirectoryGroupBox = QtGui.QGroupBox(self.leftColumn)
        self.dataDirectoryGroupBox.setObjectName(u'dataDirectoryGroupBox')
        self.dataDirectoryLayout = QtGui.QFormLayout(self.dataDirectoryGroupBox)
        self.dataDirectoryLayout.setObjectName(u'dataDirectoryLayout')
        self.dataDirectoryCurrentLabel = QtGui.QLabel(self.dataDirectoryGroupBox)
        self.dataDirectoryCurrentLabel.setObjectName(
            u'dataDirectoryCurrentLabel')
        self.dataDirectoryLabel = QtGui.QLabel(self.dataDirectoryGroupBox)
        self.dataDirectoryLabel.setObjectName(u'dataDirectoryLabel')
        self.dataDirectoryNewLabel = QtGui.QLabel(self.dataDirectoryGroupBox)
        self.dataDirectoryNewLabel.setObjectName(u'dataDirectoryCurrentLabel')
        self.newDataDirectoryEdit = QtGui.QLineEdit(self.dataDirectoryGroupBox)
        self.newDataDirectoryEdit.setObjectName(u'newDataDirectoryEdit')
        self.newDataDirectoryEdit.setReadOnly(True)
        self.newDataDirectoryHasFilesLabel = QtGui.QLabel(
            self.dataDirectoryGroupBox)
        self.newDataDirectoryHasFilesLabel.setObjectName(
            u'newDataDirectoryHasFilesLabel')
        self.newDataDirectoryHasFilesLabel.setWordWrap(True)
        self.dataDirectoryBrowseButton = QtGui.QToolButton(
            self.dataDirectoryGroupBox)
        self.dataDirectoryBrowseButton.setObjectName(
            u'dataDirectoryBrowseButton')
        self.dataDirectoryBrowseButton.setIcon(
            build_icon(u':/general/general_open.png'))
        self.dataDirectoryDefaultButton = QtGui.QToolButton(
            self.dataDirectoryGroupBox)
        self.dataDirectoryDefaultButton.setObjectName(
            u'dataDirectoryDefaultButton')
        self.dataDirectoryDefaultButton.setIcon(
            build_icon(u':/general/general_revert.png'))
        self.dataDirectoryCancelButton = QtGui.QToolButton(
            self.dataDirectoryGroupBox)
        self.dataDirectoryCancelButton.setObjectName(
            u'dataDirectoryCancelButton')
        self.dataDirectoryCancelButton.setIcon(
            build_icon(u':/general/general_delete.png'))
        self.newDataDirectoryLabelHBox = QtGui.QHBoxLayout()
        self.newDataDirectoryLabelHBox.setObjectName(
            u'newDataDirectoryLabelHBox')
        self.newDataDirectoryLabelHBox.addWidget(self.newDataDirectoryEdit)
        self.newDataDirectoryLabelHBox.addWidget(
            self.dataDirectoryBrowseButton)
        self.newDataDirectoryLabelHBox.addWidget(
            self.dataDirectoryDefaultButton)
        self.dataDirectoryCopyCheckHBox = QtGui.QHBoxLayout()
        self.dataDirectoryCopyCheckHBox.setObjectName(
            u'dataDirectoryCopyCheckHBox')
        self.dataDirectoryCopyCheckBox = QtGui.QCheckBox(
            self.dataDirectoryGroupBox)
        self.dataDirectoryCopyCheckBox.setObjectName(
            u'dataDirectoryCopyCheckBox')
        self.dataDirectoryCopyCheckHBox.addWidget(
            self.dataDirectoryCopyCheckBox)
        self.dataDirectoryCopyCheckHBox.addStretch()
        self.dataDirectoryCopyCheckHBox.addWidget(
            self.dataDirectoryCancelButton)
        self.dataDirectoryLayout.addRow(self.dataDirectoryCurrentLabel,
            self.dataDirectoryLabel)
        self.dataDirectoryLayout.addRow(self.dataDirectoryNewLabel,
            self.newDataDirectoryLabelHBox)
        self.dataDirectoryLayout.addRow(self.dataDirectoryCopyCheckHBox)
        self.dataDirectoryLayout.addRow(self.newDataDirectoryHasFilesLabel)
        self.leftLayout.addWidget(self.dataDirectoryGroupBox)
        self.leftLayout.addStretch()
        # Default Image
        self.defaultImageGroupBox = QtGui.QGroupBox(self.rightColumn)
        self.defaultImageGroupBox.setObjectName(u'defaultImageGroupBox')
        self.defaultImageLayout = QtGui.QFormLayout(self.defaultImageGroupBox)
        self.defaultImageLayout.setObjectName(u'defaultImageLayout')
        self.defaultColorLabel = QtGui.QLabel(self.defaultImageGroupBox)
        self.defaultColorLabel.setObjectName(u'defaultColorLabel')
        self.defaultColorButton = QtGui.QPushButton(self.defaultImageGroupBox)
        self.defaultColorButton.setObjectName(u'defaultColorButton')
        self.defaultImageLayout.addRow(self.defaultColorLabel,
            self.defaultColorButton)
        self.defaultFileLabel = QtGui.QLabel(self.defaultImageGroupBox)
        self.defaultFileLabel.setObjectName(u'defaultFileLabel')
        self.defaultFileEdit = QtGui.QLineEdit(self.defaultImageGroupBox)
        self.defaultFileEdit.setObjectName(u'defaultFileEdit')
        self.defaultBrowseButton = QtGui.QToolButton(self.defaultImageGroupBox)
        self.defaultBrowseButton.setObjectName(u'defaultBrowseButton')
        self.defaultBrowseButton.setIcon(
            build_icon(u':/general/general_open.png'))
        self.defaultRevertButton = QtGui.QToolButton(self.defaultImageGroupBox)
        self.defaultRevertButton.setObjectName(u'defaultRevertButton')
        self.defaultRevertButton.setIcon(
            build_icon(u':/general/general_revert.png'))
        self.defaultFileLayout = QtGui.QHBoxLayout()
        self.defaultFileLayout.setObjectName(u'defaultFileLayout')
        self.defaultFileLayout.addWidget(self.defaultFileEdit)
        self.defaultFileLayout.addWidget(self.defaultBrowseButton)
        self.defaultFileLayout.addWidget(self.defaultRevertButton)
        self.defaultImageLayout.addRow(self.defaultFileLabel,
            self.defaultFileLayout)
        self.rightLayout.addWidget(self.defaultImageGroupBox)
        # Hide mouse
        self.hideMouseGroupBox = QtGui.QGroupBox(self.rightColumn)
        self.hideMouseGroupBox.setObjectName(u'hideMouseGroupBox')
        self.hideMouseLayout = QtGui.QVBoxLayout(self.hideMouseGroupBox)
        self.hideMouseLayout.setObjectName(u'hideMouseLayout')
        self.hideMouseCheckBox = QtGui.QCheckBox(self.hideMouseGroupBox)
        self.hideMouseCheckBox.setObjectName(u'hideMouseCheckBox')
        self.hideMouseLayout.addWidget(self.hideMouseCheckBox)
        self.rightLayout.addWidget(self.hideMouseGroupBox)
        # Service Item Slide Limits
        self.slideGroupBox = QtGui.QGroupBox(self.rightColumn)
        self.slideGroupBox.setObjectName(u'slideGroupBox')
        self.slideLayout = QtGui.QVBoxLayout(self.slideGroupBox)
        self.slideLayout.setObjectName(u'slideLayout')
        self.slideLabel = QtGui.QLabel(self.slideGroupBox)
        self.slideLabel.setWordWrap(True)
        self.slideLayout.addWidget(self.slideLabel)
        self.endSlideRadioButton = QtGui.QRadioButton(self.slideGroupBox)
        self.endSlideRadioButton.setObjectName(u'endSlideRadioButton')
        self.slideLayout.addWidget(self.endSlideRadioButton)
        self.wrapSlideRadioButton = QtGui.QRadioButton(self.slideGroupBox)
        self.wrapSlideRadioButton.setObjectName(u'wrapSlideRadioButton')
        self.slideLayout.addWidget(self.wrapSlideRadioButton)
        self.nextItemRadioButton = QtGui.QRadioButton(self.slideGroupBox)
        self.nextItemRadioButton.setObjectName(u'nextItemRadioButton')
        self.slideLayout.addWidget(self.nextItemRadioButton)
        self.rightLayout.addWidget(self.slideGroupBox)
        self.x11GroupBox = QtGui.QGroupBox(self.leftColumn)
        self.x11GroupBox.setObjectName(u'x11GroupBox')
        self.x11Layout = QtGui.QVBoxLayout(self.x11GroupBox)
        self.x11Layout.setObjectName(u'x11Layout')
        self.x11BypassCheckBox = QtGui.QCheckBox(self.x11GroupBox)
        self.x11BypassCheckBox.setObjectName(u'x11BypassCheckBox')
        self.x11Layout.addWidget(self.x11BypassCheckBox)
        self.rightLayout.addWidget(self.x11GroupBox)
        self.rightLayout.addStretch()
        self.shouldUpdateServiceNameExample = False
        QtCore.QObject.connect(self.serviceNameCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.serviceNameCheckBoxToggled)
        QtCore.QObject.connect(self.serviceNameDay,
            QtCore.SIGNAL(u'currentIndexChanged(int)'),
            self.onServiceNameDayChanged)
        QtCore.QObject.connect(self.serviceNameTime,
            QtCore.SIGNAL(u'timeChanged(QTime)'),
            self.updateServiceNameExample)
        QtCore.QObject.connect(self.serviceNameEdit,
            QtCore.SIGNAL(u'textChanged(QString)'),
            self.updateServiceNameExample)
        QtCore.QObject.connect(self.serviceNameRevertButton,
            QtCore.SIGNAL(u'clicked()'),
            self.onServiceNameRevertButtonClicked)
        QtCore.QObject.connect(self.defaultColorButton,
            QtCore.SIGNAL(u'clicked()'), self.onDefaultColorButtonClicked)
        QtCore.QObject.connect(self.defaultBrowseButton,
            QtCore.SIGNAL(u'clicked()'), self.onDefaultBrowseButtonClicked)
        QtCore.QObject.connect(self.defaultRevertButton,
            QtCore.SIGNAL(u'clicked()'), self.onDefaultRevertButtonClicked)
        QtCore.QObject.connect(self.x11BypassCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'), self.onX11BypassCheckBoxToggled)
        QtCore.QObject.connect(self.dataDirectoryBrowseButton,
            QtCore.SIGNAL(u'clicked()'),
            self.onDataDirectoryBrowseButtonClicked)
        QtCore.QObject.connect(self.dataDirectoryDefaultButton,
            QtCore.SIGNAL(u'clicked()'),
            self.onDataDirectoryDefaultButtonClicked)
        QtCore.QObject.connect(self.dataDirectoryCancelButton,
            QtCore.SIGNAL(u'clicked()'),
            self.onDataDirectoryCancelButtonClicked)
        QtCore.QObject.connect(self.dataDirectoryCopyCheckBox,
            QtCore.SIGNAL(u'toggled(bool)'),
                self.onDataDirectoryCopyCheckBoxToggled)
        QtCore.QObject.connect(self.endSlideRadioButton,
            QtCore.SIGNAL(u'clicked()'), self.onEndSlideButtonClicked)
        QtCore.QObject.connect(self.wrapSlideRadioButton,
            QtCore.SIGNAL(u'clicked()'), self.onWrapSlideButtonClicked)
        QtCore.QObject.connect(self.nextItemRadioButton,
            QtCore.SIGNAL(u'clicked()'), self.onnextItemButtonClicked)

    def retranslateUi(self):
        """
        Setup the interface translation strings.
        """
        self.tabTitleVisible = UiStrings().Advanced
        self.uiGroupBox.setTitle(
            translate('OpenLP.AdvancedTab', 'UI Settings'))
        self.dataDirectoryGroupBox.setTitle(
            translate('OpenLP.AdvancedTab', 'Data Location'))
        self.recentLabel.setText(
            translate('OpenLP.AdvancedTab',
                'Number of recent files to display:'))
        self.mediaPluginCheckBox.setText(translate('OpenLP.AdvancedTab',
            'Remember active media manager tab on startup'))
        self.doubleClickLiveCheckBox.setText(translate('OpenLP.AdvancedTab',
            'Double-click to send items straight to live'))
        self.singleClickPreviewCheckBox.setText(translate('OpenLP.AdvancedTab',
            'Preview items when clicked in Media Manager'))
        self.expandServiceItemCheckBox.setText(translate('OpenLP.AdvancedTab',
            'Expand new service items on creation'))
        self.enableAutoCloseCheckBox.setText(translate('OpenLP.AdvancedTab',
            'Enable application exit confirmation'))
        self.serviceNameGroupBox.setTitle(
            translate('OpenLP.AdvancedTab', 'Default Service Name'))
        self.serviceNameCheckBox.setText(
            translate('OpenLP.AdvancedTab', 'Enable default service name'))
        self.serviceNameTimeLabel.setText(
            translate('OpenLP.AdvancedTab', 'Date and Time:'))
        self.serviceNameDay.setItemText(0,
            translate('OpenLP.AdvancedTab', 'Monday'))
        self.serviceNameDay.setItemText(1,
            translate('OpenLP.AdvancedTab', 'Tuesday'))
        self.serviceNameDay.setItemText(2,
            translate('OpenLP.AdvancedTab', 'Wednesday'))
        self.serviceNameDay.setItemText(3,
            translate('OpenLP.AdvancedTab', 'Thurdsday'))
        self.serviceNameDay.setItemText(4,
            translate('OpenLP.AdvancedTab', 'Friday'))
        self.serviceNameDay.setItemText(5,
            translate('OpenLP.AdvancedTab', 'Saturday'))
        self.serviceNameDay.setItemText(6,
            translate('OpenLP.AdvancedTab', 'Sunday'))
        self.serviceNameDay.setItemText(7,
            translate('OpenLP.AdvancedTab', 'Now'))
        self.serviceNameTime.setToolTip(translate('OpenLP.AdvancedTab',
            'Time when usual service starts.'))
        self.serviceNameLabel.setText(
            translate('OpenLP.AdvancedTab', 'Name:'))
        self.serviceNameEdit.setToolTip(translate('OpenLP.AdvancedTab',
            'Consult the OpenLP manual for usage.'))
        self.serviceNameRevertButton.setToolTip(unicode(
            translate('OpenLP.AdvancedTab',
            'Revert to the default service name "%s".')) %
            self.defaultServiceName)
        self.serviceNameExampleLabel.setText(translate('OpenLP.AdvancedTab',
            'Example:'))
        self.hideMouseGroupBox.setTitle(translate('OpenLP.AdvancedTab',
            'Mouse Cursor'))
        self.hideMouseCheckBox.setText(translate('OpenLP.AdvancedTab',
            'Hide mouse cursor when over display window'))
        self.defaultImageGroupBox.setTitle(translate('OpenLP.AdvancedTab',
            'Default Image'))
        self.defaultColorLabel.setText(translate('OpenLP.AdvancedTab',
            'Background color:'))
        self.defaultColorButton.setToolTip(translate('OpenLP.AdvancedTab',
            'Click to select a color.'))
        self.defaultFileLabel.setText(translate('OpenLP.AdvancedTab',
            'Image file:'))
        self.defaultBrowseButton.setToolTip(translate('OpenLP.AdvancedTab',
            'Browse for an image file to display.'))
        self.defaultRevertButton.setToolTip(translate('OpenLP.AdvancedTab',
            'Revert to the default OpenLP logo.'))
        self.dataDirectoryCurrentLabel.setText(translate('OpenLP.AdvancedTab',
            'Current path:'))
        self.dataDirectoryNewLabel.setText(translate('OpenLP.AdvancedTab',
            'Custom path:'))
        self.dataDirectoryBrowseButton.setToolTip(
            translate('OpenLP.AdvancedTab',
            'Browse for new data file location.'))
        self.dataDirectoryDefaultButton.setToolTip(
            translate('OpenLP.AdvancedTab',
            'Set the data location to the default.'))
        self.dataDirectoryCancelButton.setText(
            translate('OpenLP.AdvancedTab',
            'Cancel'))
        self.dataDirectoryCancelButton.setToolTip(
            translate('OpenLP.AdvancedTab',
            'Cancel OpenLP data directory location change.'))
        self.dataDirectoryCopyCheckBox.setText(
            translate('OpenLP.AdvancedTab',
            'Copy data to new location.'))
        self.dataDirectoryCopyCheckBox.setToolTip(
            translate('OpenLP.AdvancedTab',
            'Copy the OpenLP data files to the new location.'))
        self.newDataDirectoryHasFilesLabel.setText(
            translate('OpenLP.AdvancedTab',
            '<strong>WARNING:</strong> New data directory location contains '
            'OpenLP data files.  These files WILL be replaced during a copy.'))
        self.x11GroupBox.setTitle(translate('OpenLP.AdvancedTab', 'X11'))
        self.x11BypassCheckBox.setText(translate('OpenLP.AdvancedTab',
            'Bypass X11 Window Manager'))
        # Slide Limits
        self.slideGroupBox.setTitle(
            translate('OpenLP.GeneralTab', 'Service Item Slide Limits'))
        self.slideLabel.setText(translate('OpenLP.GeneralTab',
            'Behavior of next/previous on the last/first slide:'))
        self.endSlideRadioButton.setText(
            translate('OpenLP.GeneralTab', '&Remain on Slide'))
        self.wrapSlideRadioButton.setText(
            translate('OpenLP.GeneralTab', '&Wrap around'))
        self.nextItemRadioButton.setText(translate('OpenLP.GeneralTab',
            '&Move to next/previous service item'))

    def load(self):
        """
        Load settings from disk.
        """
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        # The max recent files value does not have an interface and so never
        # gets actually stored in the settings therefore the default value of
        # 20 will always be used.
        self.recentSpinBox.setMaximum(Settings().value(
            u'max recent files', QtCore.QVariant(20)).toInt()[0])
        self.recentSpinBox.setValue(settings.value(u'recent file count',
            QtCore.QVariant(4)).toInt()[0])
        self.mediaPluginCheckBox.setChecked(
            settings.value(u'save current plugin',
            QtCore.QVariant(False)).toBool())
        self.doubleClickLiveCheckBox.setChecked(
            settings.value(u'double click live',
            QtCore.QVariant(False)).toBool())
        self.singleClickPreviewCheckBox.setChecked(
            settings.value(u'single click preview',
            QtCore.QVariant(False)).toBool())
        self.expandServiceItemCheckBox.setChecked(
            settings.value(u'expand service item',
            QtCore.QVariant(False)).toBool())
        self.enableAutoCloseCheckBox.setChecked(
            settings.value(u'enable exit confirmation',
            QtCore.QVariant(True)).toBool())
        self.hideMouseCheckBox.setChecked(
            settings.value(u'hide mouse', QtCore.QVariant(False)).toBool())
        self.serviceNameDay.setCurrentIndex(
            settings.value(u'default service day',
            QtCore.QVariant(self.defaultServiceDay)).toInt()[0])
        self.serviceNameTime.setTime(QtCore.QTime(
            settings.value(u'default service hour',
            self.defaultServiceHour).toInt()[0],
            settings.value(u'default service minute',
            self.defaultServiceMinute).toInt()[0]))
        self.shouldUpdateServiceNameExample = True
        self.serviceNameEdit.setText(settings.value(u'default service name',
            self.defaultServiceName).toString())
        default_service_enabled = settings.value(u'default service enabled',
            QtCore.QVariant(True)).toBool()
        self.serviceNameCheckBox.setChecked(default_service_enabled)
        self.serviceNameCheckBoxToggled(default_service_enabled)
        # Fix for bug #1014422.
        x11_bypass_default = True
        if sys.platform.startswith(u'linux'):
            # Default to False on Gnome.
            x11_bypass_default = bool(not
                os.environ.get(u'GNOME_DESKTOP_SESSION_ID'))
            # Default to False on XFce
            if os.environ.get(u'DESKTOP_SESSION') == u'xfce':
                x11_bypass_default = False
        self.x11BypassCheckBox.setChecked(settings.value(
            u'x11 bypass wm', QtCore.QVariant(x11_bypass_default)).toBool())
        self.defaultColor = settings.value(u'default color',
            QtCore.QVariant(u'#ffffff')).toString()
        self.defaultFileEdit.setText(settings.value(u'default image',
            QtCore.QVariant(u':/graphics/openlp-splash-screen.png'))\
            .toString())
        self.slide_limits = settings.value(
            u'slide limits', QtCore.QVariant(SlideLimits.End)).toInt()[0]
        if self.slide_limits == SlideLimits.End:
            self.endSlideRadioButton.setChecked(True)
        elif self.slide_limits == SlideLimits.Wrap:
            self.wrapSlideRadioButton.setChecked(True)
        else:
            self.nextItemRadioButton.setChecked(True)
        settings.endGroup()
        self.dataDirectoryCopyCheckBox.hide()
        self.newDataDirectoryHasFilesLabel.hide()
        self.dataDirectoryCancelButton.hide()
        # Since data location can be changed, make sure the path is present.
        self.currentDataPath = AppLocation.get_data_path()
        if not os.path.exists(self.currentDataPath):
            log.error(u'Data path not found %s' % self.currentDataPath)
            answer = QtGui.QMessageBox.critical(self,
                translate('OpenLP.AdvancedTab',
                'Data Directory Error'),
                translate('OpenLP.AdvancedTab',
                'OpenLP data directory was not found\n\n%s\n\n'
                'This data directory was previously changed from the OpenLP '
                'default location.  If the new location was on removable '
                'media, that media needs to be made available.\n\n'
                'Click "No" to stop loading OpenLP. allowing you to fix '
                'the the problem.\n\n'
                'Click "Yes" to reset the data directory to the default '
                'location.').replace('%s', self.currentDataPath),
                QtGui.QMessageBox.StandardButtons(
                QtGui.QMessageBox.Yes |
                QtGui.QMessageBox.No),
                QtGui.QMessageBox.No)
            if answer == QtGui.QMessageBox.No:
                log.info(u'User requested termination')
                Receiver.send_message(u'cleanup')
                sys.exit()
            # Set data location to default.
            settings.remove(u'advanced/data path')
            self.currentDataPath = AppLocation.get_data_path()
            log.warning(u'User requested data path set to default %s'
                % self.currentDataPath)
        self.dataDirectoryLabel.setText(os.path.abspath(
            self.currentDataPath))
        self.defaultColorButton.setStyleSheet(
            u'background-color: %s' % self.defaultColor)
        # Don't allow data directory move if running portable.
        if Settings().value(u'advanced/is portable',
            QtCore.QVariant(False)).toBool():
            self.dataDirectoryGroupBox.hide()

    def save(self):
        """
        Save settings to disk.
        """
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        settings.setValue(u'default service enabled',
            self.serviceNameCheckBox.isChecked())
        service_name = unicode(self.serviceNameEdit.text())
        preset_is_valid = self.generateServiceNameExample()[0]
        if service_name == self.defaultServiceName or not preset_is_valid:
            settings.remove(u'default service name')
            self.serviceNameEdit.setText(service_name)
        else:
            settings.setValue(u'default service name', service_name)
        settings.setValue(u'default service day',
            self.serviceNameDay.currentIndex())
        settings.setValue(u'default service hour',
            self.serviceNameTime.time().hour())
        settings.setValue(u'default service minute',
            self.serviceNameTime.time().minute())
        settings.setValue(u'recent file count',
            QtCore.QVariant(self.recentSpinBox.value()))
        settings.setValue(u'save current plugin',
            QtCore.QVariant(self.mediaPluginCheckBox.isChecked()))
        settings.setValue(u'double click live',
            QtCore.QVariant(self.doubleClickLiveCheckBox.isChecked()))
        settings.setValue(u'single click preview',
            QtCore.QVariant(self.singleClickPreviewCheckBox.isChecked()))
        settings.setValue(u'expand service item',
            QtCore.QVariant(self.expandServiceItemCheckBox.isChecked()))
        settings.setValue(u'enable exit confirmation',
            QtCore.QVariant(self.enableAutoCloseCheckBox.isChecked()))
        settings.setValue(u'hide mouse',
            QtCore.QVariant(self.hideMouseCheckBox.isChecked()))
        settings.setValue(u'x11 bypass wm',
            QtCore.QVariant(self.x11BypassCheckBox.isChecked()))
        settings.setValue(u'default color', self.defaultColor)
        settings.setValue(u'default image', self.defaultFileEdit.text())
        settings.setValue(u'slide limits', QtCore.QVariant(self.slide_limits))
        settings.endGroup()
        if self.displayChanged:
            Receiver.send_message(u'config_screen_changed')
            self.displayChanged = False
        Receiver.send_message(u'slidecontroller_update_slide_limits')

    def cancel(self):
        # Dialogue was cancelled, remove any pending data path change.
        self.onDataDirectoryCancelButtonClicked()
        SettingsTab.cancel(self)

    def serviceNameCheckBoxToggled(self, default_service_enabled):
        self.serviceNameDay.setEnabled(default_service_enabled)
        time_enabled = default_service_enabled and \
            self.serviceNameDay.currentIndex() is not 7
        self.serviceNameTime.setEnabled(time_enabled)
        self.serviceNameEdit.setEnabled(default_service_enabled)
        self.serviceNameRevertButton.setEnabled(default_service_enabled)

    def generateServiceNameExample(self):
        preset_is_valid = True
        if self.serviceNameDay.currentIndex() == 7:
            local_time = datetime.now()
        else:
            now = datetime.now()
            day_delta = self.serviceNameDay.currentIndex() - now.weekday()
            if day_delta < 0:
                day_delta += 7
            time = now + timedelta(days=day_delta)
            local_time = time.replace(hour = self.serviceNameTime.time().hour(),
                minute = self.serviceNameTime.time().minute())
        try:
            service_name_example = format_time(unicode(
                self.serviceNameEdit.text()), local_time)
        except ValueError:
            preset_is_valid = False
            service_name_example = translate('OpenLP.AdvancedTab',
                'Syntax error.')
        return preset_is_valid, service_name_example

    def updateServiceNameExample(self, returned_value):
        if not self.shouldUpdateServiceNameExample:
            return
        name_example = self.generateServiceNameExample()[1]
        self.serviceNameExample.setText(name_example)

    def onServiceNameDayChanged(self, service_day):
        self.serviceNameTime.setEnabled(service_day is not 7)
        self.updateServiceNameExample(None)

    def onServiceNameRevertButtonClicked(self):
        self.serviceNameEdit.setText(self.defaultServiceName)
        self.serviceNameEdit.setFocus()

    def onDefaultColorButtonClicked(self):
        new_color = QtGui.QColorDialog.getColor(
            QtGui.QColor(self.defaultColor), self)
        if new_color.isValid():
            self.defaultColor = new_color.name()
            self.defaultColorButton.setStyleSheet(
                u'background-color: %s' % self.defaultColor)

    def onDefaultBrowseButtonClicked(self):
        file_filters = u'%s;;%s (*.*)' % (get_images_filter(),
            UiStrings().AllFiles)
        filename = QtGui.QFileDialog.getOpenFileName(self,
            translate('OpenLP.AdvancedTab', 'Open File'), '',
            file_filters)
        if filename:
            self.defaultFileEdit.setText(filename)
        self.defaultFileEdit.setFocus()

    def onDataDirectoryBrowseButtonClicked(self):
        """
        Browse for a new data directory location.
        """
        old_root_path = unicode(self.dataDirectoryLabel.text())
        # Get the new directory location.
        new_data_path = unicode(QtGui.QFileDialog.getExistingDirectory(self,
            translate('OpenLP.AdvancedTab',
            'Select Data Directory Location'), old_root_path,
            options = QtGui.QFileDialog.ShowDirsOnly))
        # Set the new data path.
        if new_data_path:
            new_data_path = os.path.normpath(new_data_path)
            if self.currentDataPath.lower() == new_data_path.lower():
                self.onDataDirectoryCancelButtonClicked()
                return
        else:
            return
        # Make sure they want to change the data.
        answer = QtGui.QMessageBox.question(self,
            translate('OpenLP.AdvancedTab', 'Confirm Data Directory Change'),
            translate('OpenLP.AdvancedTab',
                'Are you sure you want to change the location of the OpenLP '
                'data directory to:\n\n%s\n\n'
                'The data directory will be changed when OpenLP is closed.'
                ).replace('%s', new_data_path),
            QtGui.QMessageBox.StandardButtons(
            QtGui.QMessageBox.Yes |
            QtGui.QMessageBox.No),
            QtGui.QMessageBox.No)
        if answer != QtGui.QMessageBox.Yes:
            return
        # Check if data already exists here.
        self.checkDataOverwrite(new_data_path)
        # Save the new location.
        Receiver.send_message(u'set_new_data_path', new_data_path)
        self.newDataDirectoryEdit.setText(new_data_path)
        self.dataDirectoryCancelButton.show()

    def onDataDirectoryDefaultButtonClicked(self):
        """
        Re-set the data directory location to the 'default' location.
        """
        new_data_path = AppLocation.get_directory(AppLocation.DataDir)
        if self.currentDataPath.lower() != new_data_path.lower():
            # Make sure they want to change the data location back to the default.
            answer = QtGui.QMessageBox.question(self,
                translate('OpenLP.AdvancedTab', 'Reset Data Directory'),
                translate('OpenLP.AdvancedTab',
                'Are you sure you want to change the location of the OpenLP '
                'data directory to the default location?\n\n'
                'This location will be used after OpenLP is closed.'),
                QtGui.QMessageBox.StandardButtons(
                QtGui.QMessageBox.Yes |
                QtGui.QMessageBox.No),
                QtGui.QMessageBox.No)
            if answer != QtGui.QMessageBox.Yes:
                return
            self.checkDataOverwrite(new_data_path)
            # Save the new location.
            Receiver.send_message(u'set_new_data_path', new_data_path)
            self.newDataDirectoryEdit.setText(os.path.abspath(new_data_path))
            self.dataDirectoryCancelButton.show()
        else:
            # We cancel the change in case user changed their mind.
            self.onDataDirectoryCancelButtonClicked()

    def onDataDirectoryCopyCheckBoxToggled(self):
        Receiver.send_message(u'set_copy_data',
            self.dataDirectoryCopyCheckBox.isChecked())
        if self.dataExists:
            if self.dataDirectoryCopyCheckBox.isChecked():
                self.newDataDirectoryHasFilesLabel.show()
            else:
                self.newDataDirectoryHasFilesLabel.hide()

    def checkDataOverwrite(self, data_path ):
        test_path = os.path.join(data_path, u'songs')
        self.dataDirectoryCopyCheckBox.show()
        if os.path.exists(test_path):
            self.dataExists = True
            # Check is they want to replace existing data.
            answer = QtGui.QMessageBox.warning(self,
                translate('OpenLP.AdvancedTab', 'Overwrite Existing Data'),
                translate('OpenLP.AdvancedTab',
                'WARNING: \n\n'
                'The location you have selected \n\n%s\n\n'
                'appears to contain OpenLP data files.  Do you wish to replace '
                'these files with the current data files?'
                ).replace('%s', os.path.abspath(data_path,)),
                QtGui.QMessageBox.StandardButtons(
                QtGui.QMessageBox.Yes |
                QtGui.QMessageBox.No),
                QtGui.QMessageBox.No)
            if answer == QtGui.QMessageBox.Yes:
                self.dataDirectoryCopyCheckBox.setChecked(True)
                self.newDataDirectoryHasFilesLabel.show()
            else:
                self.dataDirectoryCopyCheckBox.setChecked(False)
                self.newDataDirectoryHasFilesLabel.hide()
        else:
            self.dataExists = False
            self.dataDirectoryCopyCheckBox.setChecked(True)
            self.newDataDirectoryHasFilesLabel.hide()

    def onDataDirectoryCancelButtonClicked(self):
        """
        Cancel the data directory location change
        """
        self.newDataDirectoryEdit.clear()
        self.dataDirectoryCopyCheckBox.setChecked(False)
        Receiver.send_message(u'set_new_data_path', u'')
        Receiver.send_message(u'set_copy_data', False)
        self.dataDirectoryCopyCheckBox.hide()
        self.dataDirectoryCancelButton.hide()
        self.newDataDirectoryHasFilesLabel.hide()

    def onDefaultRevertButtonClicked(self):
        self.defaultFileEdit.setText(u':/graphics/openlp-splash-screen.png')
        self.defaultFileEdit.setFocus()

    def onX11BypassCheckBoxToggled(self, checked):
        """
        Toggle X11 bypass flag on maindisplay depending on check box state.

        ``checked``
            The state of the check box (boolean).
        """
        self.displayChanged = True

    def onEndSlideButtonClicked(self):
        self.slide_limits = SlideLimits.End

    def onWrapSlideButtonClicked(self):
        self.slide_limits = SlideLimits.Wrap

    def onnextItemButtonClicked(self):
        self.slide_limits = SlideLimits.Next



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging
import re

from lxml import etree, objectify

from openlp.plugins.songs.lib import VerseType
from openlp.plugins.songs.lib.songimport import SongImport

log = logging.getLogger(__name__)

class EasySlidesImport(SongImport):
    """
    Import songs exported from EasySlides

    The format example is here:
    http://wiki.openlp.org/Development:EasySlides_-_Song_Data_Format
    """
    def __init__(self, manager, **kwargs):
        """
        Initialise the class.
        """
        SongImport.__init__(self, manager, **kwargs)

    def doImport(self):
        log.info(u'Importing EasySlides XML file %s', self.importSource)
        parser = etree.XMLParser(remove_blank_text=True)
        parsed_file = etree.parse(self.importSource, parser)
        xml = unicode(etree.tostring(parsed_file))
        song_xml = objectify.fromstring(xml)
        self.importWizard.progressBar.setMaximum(len(song_xml.Item))
        for song in song_xml.Item:
            if self.stopImportFlag:
                return
            self._parseSong(song)

    def _parseSong(self, song):
        self._success = True
        self._addUnicodeAttribute(u'title', song.Title1, True)
        if hasattr(song, u'Title2'):
            self._addUnicodeAttribute(u'alternateTitle', song.Title2)
        if hasattr(song, u'SongNumber'):
            self._addUnicodeAttribute(u'songNumber', song.SongNumber)
        if self.songNumber == u'0':
            self.songNumber = u''
        self._addAuthors(song)
        if hasattr(song, u'Copyright'):
            self._addCopyright(song.Copyright)
        if hasattr(song, u'LicenceAdmin1'):
            self._addCopyright(song.LicenceAdmin1)
        if hasattr(song, u'LicenceAdmin2'):
            self._addCopyright(song.LicenceAdmin2)
        if hasattr(song, u'BookReference'):
            self._addUnicodeAttribute(u'songBookName', song.BookReference)
        self._parseAndAddLyrics(song)
        if self._success:
            if not self.finish():
                self.logError(song.Title1 if song.Title1 else u'')
        else:
            self.setDefaults()

    def _addUnicodeAttribute(self, self_attribute, import_attribute,
        mandatory=False):
        """
        Add imported values to the song model converting them to unicode at the
        same time. If the unicode decode fails or a mandatory attribute is not
        present _success is set to False so the importer can react
        appropriately.

        ``self_attribute``
            The attribute in the song model to populate.

        ``import_attribute``
            The imported value to convert to unicode and save to the song.

        ``mandatory``
            Signals that this attribute must exist in a valid song.
        """
        try:
            setattr(self, self_attribute, unicode(import_attribute).strip())
        except UnicodeDecodeError:
            log.exception(u'UnicodeDecodeError decoding %s' % import_attribute)
            self._success = False
        except AttributeError:
            log.exception(u'No attribute %s' % import_attribute)
            if mandatory:
                self._success = False

    def _addAuthors(self, song):
        try:
            authors = unicode(song.Writer).split(u',')
            self.authors = \
                [author.strip() for author in authors if author.strip()]
        except UnicodeDecodeError:
            log.exception(u'Unicode decode error while decoding Writer')
            self._success = False
        except AttributeError:
            pass

    def _addCopyright(self, element):
        """
        Add a piece of copyright to the total copyright information for the
        song.

        ``element``
            The imported variable to get the data from.
        """
        try:
            self.addCopyright(unicode(element).strip())
        except UnicodeDecodeError:
            log.exception(u'Unicode error on decoding copyright: %s' % element)
            self._success = False
        except AttributeError:
            pass

    def _parseAndAddLyrics(self, song):
        try:
            lyrics = unicode(song.Contents).strip()
        except UnicodeDecodeError:
            log.exception(u'Unicode decode error while decoding Contents')
            self._success = False
        except AttributeError:
            log.exception(u'no Contents')
            self._success = False
        lines = lyrics.split(u'\n')
        # we go over all lines first, to determine information,
        # which tells us how to parse verses later
        regionlines = {}
        separatorlines = 0
        for line in lines:
            line = line.strip()
            if not line:
                continue
            elif line[1:7] == u'region':
                # this is region separator, probably [region 2]
                region = self._extractRegion(line)
                regionlines[region] = 1 + regionlines.get(region, 0)
            elif line[0] == u'[':
                separatorlines = separatorlines + 1
        # if the song has separators
        separators = (separatorlines > 0)
        # the number of different regions in song - 1
        if len(regionlines) > 1:
            log.info(u'EasySlidesImport: the file contained a song named "%s"'
                u'with more than two regions, but only two regions are',
                u'tested, encountered regions were: %s',
                self.title, u','.join(regionlines.keys()))
        # if the song has regions
        regions = (len(regionlines) > 0)
        # if the regions are inside verses
        regionsInVerses = (regions and regionlines[regionlines.keys()[0]] > 1)
        MarkTypes = {
            u'CHORUS': VerseType.Tags[VerseType.Chorus],
            u'VERSE': VerseType.Tags[VerseType.Verse],
            u'INTRO': VerseType.Tags[VerseType.Intro],
            u'ENDING': VerseType.Tags[VerseType.Ending],
            u'BRIDGE': VerseType.Tags[VerseType.Bridge],
            u'PRECHORUS': VerseType.Tags[VerseType.PreChorus]
        }
        verses = {}
        # list as [region, versetype, versenum, instance]
        our_verse_order = []
        defaultregion = u'1'
        reg = defaultregion
        verses[reg] = {}
        # instance differentiates occurrences of same verse tag
        vt = u'V'
        vn = u'1'
        inst = 1

        for line in lines:
            line = line.strip()
            if not line:
                if separators:
                    # separators are used, so empty line means slide break
                    # inside verse
                    if self._listHas(verses, [reg, vt, vn, inst]):
                        inst = inst + 1
                else:
                    # separators are not used, so empty line starts a new verse
                    vt = u'V'
                    vn = len(verses[reg].get(vt, {})) + 1
                    inst = 1
            elif line[0:7] == u'[region':
                reg = self._extractRegion(line)
                verses.setdefault(reg, {})
                if not regionsInVerses:
                    vt = u'V'
                    vn = u'1'
                    inst = 1
            elif line[0] == u'[':
                # this is a normal section marker
                marker = line[1:line.find(u']')].upper()
                vn = u'1'
                # have we got any digits?
                # If so, versenumber is everything from the digits to the end
                match = re.match(u'(.*)(\d+.*)', marker)
                if match:
                    marker = match.group(1).strip()
                    vn = match.group(2)
                vt = MarkTypes.get(marker, u'O') if marker else u'V'
                if regionsInVerses:
                    region = defaultregion
                inst = 1
                if self._listHas(verses, [reg, vt, vn, inst]):
                    inst = len(verses[reg][vt][vn]) + 1
            else:
                if not [reg, vt, vn, inst] in our_verse_order:
                    our_verse_order.append([reg, vt, vn, inst])
                verses[reg].setdefault(vt, {})
                verses[reg][vt].setdefault(vn, {})
                verses[reg][vt][vn].setdefault(inst, [])
                verses[reg][vt][vn][inst].append(self.tidyText(line))
        # done parsing

        versetags = []
        # we use our_verse_order to ensure, we insert lyrics in the same order
        # as these appeared originally in the file
        for [reg, vt, vn, inst] in our_verse_order:
            if self._listHas(verses, [reg, vt, vn, inst]):
                # this is false, but needs user input
                lang = None
                versetag = u'%s%s' % (vt, vn)
                versetags.append(versetag)
                lines = u'\n'.join(verses[reg][vt][vn][inst])
                self.verses.append([versetag, lines, lang])

        SeqTypes = {
            u'p': u'P1',
            u'q': u'P2',
            u'c': u'C1',
            u't': u'C2',
            u'b': u'B1',
            u'w': u'B2',
            u'e': u'E1'}
        # Make use of Sequence data, determining the order of verses
        try:
            order = unicode(song.Sequence).strip().split(u',')
            for tag in order:
                if not tag:
                    continue
                elif tag[0].isdigit():
                    tag = u'V' + tag
                elif tag.lower() in SeqTypes:
                    tag = SeqTypes[tag.lower()]
                else:
                    continue
                if tag in versetags:
                    self.verseOrderList.append(tag)
                else:
                    log.info(u'Got order item %s, which is not in versetags,'
                        u'dropping item from presentation order', tag)
        except UnicodeDecodeError:
            log.exception(u'Unicode decode error while decoding Sequence')
            self._success = False
        except AttributeError:
            pass

    def _listHas(self, lst, subitems):
        for subitem in subitems:
            if subitem in lst:
                lst = lst[subitem]
            else:
                return False
        return True

    def _extractRegion(self, line):
        # this was true already: line[0:7] == u'[region':
        right_bracket = line.find(u']')
        return line[7:right_bracket].strip()



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtGui

from openlp.core.lib import translate, SpellTextEdit, build_icon
from openlp.core.lib.ui import UiStrings, create_button, create_button_box


class Ui_CustomSlideEditDialog(object):
    def setupUi(self, customSlideEditDialog):
        customSlideEditDialog.setObjectName(u'customSlideEditDialog')
        customSlideEditDialog.resize(350, 300)
        customSlideEditDialog.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        self.dialogLayout = QtGui.QVBoxLayout(customSlideEditDialog)
        self.slideTextEdit = SpellTextEdit(self)
        self.slideTextEdit.setObjectName(u'slideTextEdit')
        self.dialogLayout.addWidget(self.slideTextEdit)
        self.splitButton = create_button(customSlideEditDialog, u'splitButton',
            icon=u':/general/general_add.png')
        self.insertButton = create_button(customSlideEditDialog,
            u'insertButton', icon=u':/general/general_add.png')
        self.buttonBox = create_button_box(customSlideEditDialog, u'buttonBox',
            [u'cancel', u'save'], [self.splitButton, self.insertButton])
        self.dialogLayout.addWidget(self.buttonBox)
        self.retranslateUi(customSlideEditDialog)

    def retranslateUi(self, customSlideEditDialog):
        customSlideEditDialog.setWindowTitle(
            translate('CustomPlugin.EditVerseForm', 'Edit Slide'))
        self.splitButton.setText(UiStrings().Split)
        self.splitButton.setToolTip(UiStrings().SplitToolTip)
        self.insertButton.setText(
            translate('CustomPlugin.EditCustomForm', 'Insert Slide'))
        self.insertButton.setToolTip(
            translate('CustomPlugin.EditCustomForm', 'Split a slide into two '
            'by inserting a slide splitter.'))



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import translate, build_icon
from openlp.core.lib.ui import create_button_box


class Ui_MediaFilesDialog(object):
    def setupUi(self, mediaFilesDialog):
        mediaFilesDialog.setObjectName(u'mediaFilesDialog')
        mediaFilesDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        mediaFilesDialog.resize(400, 300)
        mediaFilesDialog.setModal(True)
        mediaFilesDialog.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        self.filesVerticalLayout = QtGui.QVBoxLayout(mediaFilesDialog)
        self.filesVerticalLayout.setSpacing(8)
        self.filesVerticalLayout.setMargin(8)
        self.filesVerticalLayout.setObjectName(u'filesVerticalLayout')
        self.selectLabel = QtGui.QLabel(mediaFilesDialog)
        self.selectLabel.setWordWrap(True)
        self.selectLabel.setObjectName(u'selectLabel')
        self.filesVerticalLayout.addWidget(self.selectLabel)
        self.fileListWidget = QtGui.QListWidget(mediaFilesDialog)
        self.fileListWidget.setAlternatingRowColors(True)
        self.fileListWidget.setSelectionMode(
            QtGui.QAbstractItemView.ExtendedSelection)
        self.fileListWidget.setObjectName(u'fileListWidget')
        self.filesVerticalLayout.addWidget(self.fileListWidget)
        self.buttonBox = create_button_box(mediaFilesDialog, u'buttonBox',
            [u'cancel', u'ok'])
        self.filesVerticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(mediaFilesDialog)

    def retranslateUi(self, mediaFilesDialog):
        mediaFilesDialog.setWindowTitle(
            translate('SongsPlugin.MediaFilesForm', 'Select Media File(s)'))
        self.selectLabel.setText(
            translate('SongsPlugin.MediaFilesForm', u'Select one or more '
                'audio files from the list below, and click OK to import them '
                'into this song.'))




# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

"""
Module implementing BookNameForm.
"""
import logging
import re

from PyQt4.QtGui import QDialog
from PyQt4 import QtCore

from openlp.core.lib import translate
from openlp.core.lib.ui import critical_error_message_box
from openlp.plugins.bibles.forms.booknamedialog import \
    Ui_BookNameDialog
from openlp.plugins.bibles.lib import BibleStrings
from openlp.plugins.bibles.lib.db import BiblesResourcesDB

log = logging.getLogger(__name__)

class BookNameForm(QDialog, Ui_BookNameDialog):
    """
    Class to manage a dialog which help the user to refer a book name a
    to a english book name
    """
    log.info(u'BookNameForm loaded')

    def __init__(self, parent = None):
        """
        Constructor
        """
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.customSignals()
        self.book_names = BibleStrings().BookNames
        self.book_id = False

    def customSignals(self):
        """
        Set up the signals used in the booknameform.
        """
        QtCore.QObject.connect(self.oldTestamentCheckBox,
            QtCore.SIGNAL(u'stateChanged(int)'),
            self.onCheckBoxIndexChanged)
        QtCore.QObject.connect(self.newTestamentCheckBox,
            QtCore.SIGNAL(u'stateChanged(int)'),
            self.onCheckBoxIndexChanged)
        QtCore.QObject.connect(self.apocryphaCheckBox,
            QtCore.SIGNAL(u'stateChanged(int)'),
            self.onCheckBoxIndexChanged)

    def onCheckBoxIndexChanged(self, index):
        """
        Reload Combobox if CheckBox state has changed
        """
        self.reloadComboBox()

    def reloadComboBox(self):
        """
        Reload the Combobox items
        """
        self.correspondingComboBox.clear()
        items = BiblesResourcesDB.get_books()
        for item in items:
            addBook = True
            for book in self.books:
                if book.book_reference_id == item[u'id']:
                    addBook = False
                    break
            if self.oldTestamentCheckBox.checkState() == QtCore.Qt.Unchecked \
                and item[u'testament_id'] == 1:
                addBook = False
            elif self.newTestamentCheckBox.checkState() == QtCore.Qt.Unchecked \
                and item[u'testament_id'] == 2:
                addBook = False
            elif self.apocryphaCheckBox.checkState() == QtCore.Qt.Unchecked \
                and item[u'testament_id'] == 3:
                addBook = False
            if addBook:
                self.correspondingComboBox.addItem(
                    self.book_names[item[u'abbreviation']])

    def exec_(self, name, books, maxbooks):
        self.books = books
        log.debug(maxbooks)
        if maxbooks <= 27:
            self.oldTestamentCheckBox.setCheckState(QtCore.Qt.Unchecked)
            self.apocryphaCheckBox.setCheckState(QtCore.Qt.Unchecked)
        elif maxbooks <= 66:
            self.apocryphaCheckBox.setCheckState(QtCore.Qt.Unchecked)
        self.reloadComboBox()
        self.currentBookLabel.setText(unicode(name))
        self.correspondingComboBox.setFocus()
        return QDialog.exec_(self)

    def accept(self):
        if self.correspondingComboBox.currentText() == u'':
            critical_error_message_box(
                message=translate('BiblesPlugin.BookNameForm',
                'You need to select a book.'))
            self.correspondingComboBox.setFocus()
            return False
        else:
            cor_book = unicode(self.correspondingComboBox.currentText())
            for character in u'\\.^$*+?{}[]()':
                cor_book = cor_book.replace(character, u'\\' + character)
            books = filter(lambda key:
                re.match(cor_book, unicode(self.book_names[key]), re.UNICODE),
                self.book_names.keys())
            books = filter(None, map(BiblesResourcesDB.get_book, books))
            if books:
                self.book_id = books[0][u'id']
            return QDialog.accept(self)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from PyQt4 import QtCore, QtGui

from openlp.core.lib import build_icon
from openlp.core.lib.ui import create_widget_action

log = logging.getLogger(__name__)

class SearchEdit(QtGui.QLineEdit):
    """
    This is a specialised QLineEdit with a "clear" button inside for searches.
    """

    def __init__(self, parent):
        """
        Constructor.
        """
        QtGui.QLineEdit.__init__(self, parent)
        self._currentSearchType = -1
        self.clearButton = QtGui.QToolButton(self)
        self.clearButton.setIcon(build_icon(u':/system/clear_shortcut.png'))
        self.clearButton.setCursor(QtCore.Qt.ArrowCursor)
        self.clearButton.setStyleSheet(
            u'QToolButton { border: none; padding: 0px; }')
        self.clearButton.resize(18, 18)
        self.clearButton.hide()
        QtCore.QObject.connect(
            self.clearButton,
            QtCore.SIGNAL(u'clicked()'),
            self._onClearButtonClicked
        )
        QtCore.QObject.connect(
            self,
            QtCore.SIGNAL(u'textChanged(const QString&)'),
            self._onSearchEditTextChanged
        )
        self._updateStyleSheet()
        self.setAcceptDrops(False)

    def _updateStyleSheet(self):
        """
        Internal method to update the stylesheet depending on which widgets are
        available and visible.
        """
        frameWidth = self.style().pixelMetric(
            QtGui.QStyle.PM_DefaultFrameWidth)
        rightPadding = self.clearButton.width() + frameWidth
        if hasattr(self, u'menuButton'):
            leftPadding = self.menuButton.width()
            self.setStyleSheet(
                u'QLineEdit { padding-left: %spx; padding-right: %spx; } ' %
                (leftPadding, rightPadding))
        else:
            self.setStyleSheet(u'QLineEdit { padding-right: %spx; } ' %
                rightPadding)
        msz = self.minimumSizeHint()
        self.setMinimumSize(
            max(msz.width(),
                self.clearButton.width() + (frameWidth * 2) + 2),
            max(msz.height(),
                self.clearButton.height() + (frameWidth * 2) + 2)
        )

    def resizeEvent(self, event):
        """
        Reimplemented method to react to resizing of the widget.

        ``event``
            The event that happened.
        """
        size = self.clearButton.size()
        frameWidth = self.style().pixelMetric(
            QtGui.QStyle.PM_DefaultFrameWidth)
        self.clearButton.move(self.rect().right() - frameWidth - size.width(),
            (self.rect().bottom() + 1 - size.height()) / 2)
        if hasattr(self, u'menuButton'):
            size = self.menuButton.size()
            self.menuButton.move(self.rect().left() + frameWidth + 2,
                (self.rect().bottom() + 1 - size.height()) / 2)

    def currentSearchType(self):
        """
        Readonly property to return the current search type.
        """
        return self._currentSearchType

    def setCurrentSearchType(self, identifier):
        """
        Set a new current search type.

        ``identifier``
            The search type identifier (int).
        """
        menu = self.menuButton.menu()
        for action in menu.actions():
            if identifier == action.data().toInt()[0]:
                # setPlaceholderText has been implemented in Qt 4.7 and in at
                # least PyQt 4.9 (I am not sure, if it was implemented in
                # PyQt 4.8).
                try:
                    self.setPlaceholderText(action.placeholderText)
                except AttributeError:
                    pass
                self.menuButton.setDefaultAction(action)
                self._currentSearchType = identifier
                self.emit(QtCore.SIGNAL(u'searchTypeChanged(int)'), identifier)
                return True

    def setSearchTypes(self, items):
        """
        A list of tuples to be used in the search type menu. The first item in
        the list will be preselected as the default.

        ``items``
            The list of tuples to use. The tuples should contain an integer
            identifier, an icon (QIcon instance or string) and a title for the
            item in the menu. In short, they should look like this::

                (<identifier>, <icon>, <title>, <place holder text>)

            For instance::

                (1, <QIcon instance>, "Titles", "Search Song Titles...")

            Or::

                (2, ":/songs/authors.png", "Authors", "Search Authors...")
        """
        menu = QtGui.QMenu(self)
        first = None
        for identifier, icon, title, placeholder in items:
            action = create_widget_action(menu, text=title, icon=icon,
                data=identifier, triggers=self._onMenuActionTriggered)
            action.placeholderText = placeholder
            if first is None:
                first = action
                self._currentSearchType = identifier
        if not hasattr(self, u'menuButton'):
            self.menuButton = QtGui.QToolButton(self)
            self.menuButton.setIcon(build_icon(u':/system/clear_shortcut.png'))
            self.menuButton.setCursor(QtCore.Qt.ArrowCursor)
            self.menuButton.setPopupMode(QtGui.QToolButton.InstantPopup)
            self.menuButton.setStyleSheet(
                u'QToolButton { border: none; padding: 0px 10px 0px 0px; }')
            self.menuButton.resize(QtCore.QSize(28, 18))
        self.menuButton.setMenu(menu)
        self.menuButton.setDefaultAction(first)
        self.menuButton.show()
        self._updateStyleSheet()

    def _onSearchEditTextChanged(self, text):
        """
        Internally implemented slot to react to when the text in the line edit
        has changed so that we can show or hide the clear button.

        ``text``
            A :class:`~PyQt4.QtCore.QString` instance which represents the text
            in the line edit.
        """
        self.clearButton.setVisible(not text.isEmpty())

    def _onClearButtonClicked(self):
        """
        Internally implemented slot to react to the clear button being clicked
        to clear the line edit. Once it has cleared the line edit, it emits the
        ``cleared()`` signal so that an application can react to the clearing
        of the line edit.
        """
        self.clear()
        self.emit(QtCore.SIGNAL(u'cleared()'))

    def _onMenuActionTriggered(self):
        """
        Internally implemented slot to react to the select of one of the search
        types in the menu. Once it has set the correct action on the button,
        and set the current search type (using the list of identifiers provided
        by the developer), the ``searchTypeChanged(int)`` signal is emitted
        with the identifier.
        """
        sender = self.sender()
        for action in self.menuButton.menu().actions():
            action.setChecked(False)
        self.menuButton.setDefaultAction(sender)
        self._currentSearchType = sender.data().toInt()[0]
        # setPlaceholderText has been implemented in Qt 4.7 and in at least
        # PyQt 4.9 (I am not sure, if it was implemented in PyQt 4.8).
        try:
            self.setPlaceholderText(
                self.menuButton.defaultAction().placeholderText)
        except AttributeError:
            pass
        self.emit(QtCore.SIGNAL(u'searchTypeChanged(int)'),
            self._currentSearchType)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
"""
Provide handling for persisting OpenLP settings.  OpenLP uses QSettings to
manage settings persistence.  QSettings provides a single API for saving and
retrieving settings from the application but writes to disk in an OS dependant
format.
"""
import os

from PyQt4 import QtCore

from openlp.core.lib.settings import Settings
from openlp.core.utils import AppLocation

class SettingsManager(object):
    """
    Class to provide helper functions for the loading and saving of application
    settings.
    """

    @staticmethod
    def get_last_dir(section, num=None):
        """
        Read the last directory used for plugin.

        ``section``
            The section of code calling the method. This is used in the
            settings key.

        ``num``
            Defaults to *None*. A further qualifier.
        """
        if num:
            name = u'last directory %d' % num
        else:
            name = u'last directory'
        last_dir = unicode(Settings().value(
            section + u'/' + name, QtCore.QVariant(u'')).toString())
        return last_dir

    @staticmethod
    def set_last_dir(section, directory, num=None):
        """
        Save the last directory used for plugin.

        ``section``
            The section of code calling the method. This is used in the
            settings key.

        ``directory``
            The directory being stored in the settings.

        ``num``
            Defaults to *None*. A further qualifier.
        """
        if num:
            name = u'last directory %d' % num
        else:
            name = u'last directory'
        Settings().setValue(
            section + u'/' + name, QtCore.QVariant(directory))

    @staticmethod
    def set_list(section, name, list):
        """
        Save a list to application settings.

        ``section``
            The section of the settings to store this list.

        ``name``
            The name of the list to save.

        ``list``
            The list of values to save.
        """
        settings = Settings()
        settings.beginGroup(section)
        old_count = settings.value(
            u'%s count' % name, QtCore.QVariant(0)).toInt()[0]
        new_count = len(list)
        settings.setValue(u'%s count' % name, QtCore.QVariant(new_count))
        for counter in range(new_count):
            settings.setValue(
                u'%s %d' % (name, counter), QtCore.QVariant(list[counter-1]))
        if old_count > new_count:
            # Tidy up any old list items
            for counter in range(new_count, old_count):
                settings.remove(u'%s %d' % (name, counter))
        settings.endGroup()

    @staticmethod
    def load_list(section, name):
        """
        Load a list from the config file.

        ``section``
            The section of the settings to load the list from.

        ``name``
            The name of the list.
        """
        settings = Settings()
        settings.beginGroup(section)
        list_count = settings.value(
            u'%s count' % name, QtCore.QVariant(0)).toInt()[0]
        list = []
        if list_count:
            for counter in range(list_count):
                item = unicode(
                    settings.value(u'%s %d' % (name, counter)).toString())
                if item:
                    list.append(item)
        settings.endGroup()
        return list

    @staticmethod
    def get_files(section=None, extension=None):
        """
        Get a list of files from the data files path.

        ``section``
            Defaults to *None*. The section of code getting the files - used
            to load from a section's data subdirectory.

        ``extension``
            Defaults to *None*. The extension to search for.
        """
        path = AppLocation.get_data_path()
        if section:
            path = os.path.join(path, section)
        try:
            files = os.listdir(path)
        except OSError:
            return []
        if extension:
            return [filename for filename in files
                if extension == os.path.splitext(filename)[1]]
        else:
            # no filtering required
            return files



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

# OOo API documentation:
# http://wiki.services.openoffice.org/wiki/Documentation/BASIC_Guide/Structure_of_Text_Documents
# http://wiki.services.openoffice.org/wiki/Documentation/DevGuide/Text/Iterating_over_Text
# http://www.oooforum.org/forum/viewtopic.phtml?t=14409
# http://wiki.services.openoffice.org/wiki/Python

import logging
import os
import re

from oooimport import OooImport


log = logging.getLogger(__name__)

if os.name == u'nt':
    from oooimport import PAGE_BEFORE, PAGE_AFTER, PAGE_BOTH
    RuntimeException = Exception
else:
    try:
        from com.sun.star.style.BreakType import PAGE_BEFORE, PAGE_AFTER, \
            PAGE_BOTH
        from com.sun.star.uno import RuntimeException
    except ImportError:
        pass
try:
    from com.sun.star.awt.FontWeight import BOLD
except ImportError:
    BOLD = 150.0
try:
    from com.sun.star.awt.FontSlant import ITALIC
except ImportError:
    ITALIC = 2

class SofImport(OooImport):
    """
    Import songs provided on disks with the Songs of Fellowship music books
    VOLS1_2.RTF, sof3words.rtf and sof4words.rtf

    Use OpenOffice.org Writer for processing the rtf file

    The three books are not only inconsistant with each other, they are
    inconsistant in themselves too with their formatting. Not only this, but
    the 1+2 book does not space out verses correctly. This script attempts
    to sort it out, but doesn't get it 100% right. But better than having to
    type them all out!

    It attempts to detect italiced verses, and treats these as choruses in
    the verse ordering. Again not perfect, but a start.
    """
    def __init__(self, manager, **kwargs):
        """
        Initialise the class. Requires a songmanager class which is passed
        to SongImport for writing song to disk
        """
        OooImport.__init__(self, manager, **kwargs)
        self.song = False

    def processOooDocument(self):
        """
        Handle the import process for SoF files.
        """
        self.processSofFile()

    def processSofFile(self):
        """
        Process the RTF file, a paragraph at a time
        """
        self.blankLines = 0
        self.newSong()
        try:
            paragraphs = self.document.getText().createEnumeration()
            while paragraphs.hasMoreElements():
                if self.stopImportFlag:
                    return
                paragraph = paragraphs.nextElement()
                if paragraph.supportsService("com.sun.star.text.Paragraph"):
                    self.processParagraph(paragraph)
        except RuntimeException as exc:
            log.exception(u'Error processing file: %s', exc)
        if not self.finish():
            self.logError(self.filepath)

    def processParagraph(self, paragraph):
        """
        Process a paragraph.
        In the first book, a paragraph is a single line. In the latter ones
        they may contain multiple lines.
        Each paragraph contains textportions. Each textportion has it's own
        styling, e.g. italics, bold etc.
        Also check for page breaks, which indicates a new song in books 1+2.
        In later books, there may not be line breaks, so check for 3 or more
        newlines
        """
        text = u''
        textportions = paragraph.createEnumeration()
        while textportions.hasMoreElements():
            textportion = textportions.nextElement()
            if textportion.BreakType in (PAGE_BEFORE, PAGE_BOTH):
                self.processParagraphText(text)
                self.newSong()
                text = u''
            text += self.processTextPortion(textportion)
            if textportion.BreakType in (PAGE_AFTER, PAGE_BOTH):
                self.processParagraphText(text)
                self.newSong()
                text = u''
        self.processParagraphText(text)

    def processParagraphText(self, text):
        """
        Split the paragraph text into multiple lines and process
        """
        for line in text.split(u'\n'):
            self.processParagraphLine(line)
        if self.blankLines > 2:
            self.newSong()

    def processParagraphLine(self, text):
        """
        Process a single line. Throw away that text which isn't relevant, i.e.
        stuff that appears at the end of the song.
        Anything that is OK, append to the current verse
        """
        text = text.strip()
        if text == u'':
            self.blankLines += 1
            if self.blankLines > 1:
                return
            if self.title != u'':
                self.finishVerse()
            return
        self.blankLines = 0
        if self.skipToCloseBracket:
            if text.endswith(u')'):
                self.skipToCloseBracket = False
            return
        if text.startswith(u'CCL Licence'):
            self.italics = False
            return
        if text == u'A Songs of Fellowship Worship Resource':
            return
        if text.startswith(u'(NB.') or text.startswith(u'(Regrettably') \
            or text.startswith(u'(From'):
            self.skipToCloseBracket = True
            return
        if text.startswith(u'Copyright'):
            self.addCopyright(text)
            return
        if text == u'(Repeat)':
            self.finishVerse()
            self.repeatVerse()
            return
        if self.title == u'':
            if self.copyright == u'':
                self.addSofAuthor(text)
            else:
                self.addCopyright(text)
            return
        self.addVerseLine(text)

    def processTextPortion(self, textportion):
        """
        Process a text portion. Here we just get the text and detect if
        it's bold or italics. If it's bold then its a song number or song title.
        Song titles are in all capitals, so we must bring the capitalization
        into line
        """
        text = textportion.getString()
        text = self.tidyText(text)
        if text.strip() == u'':
            return text
        if textportion.CharWeight == BOLD:
            boldtext = text.strip()
            if boldtext.isdigit() and self.songNumber == '':
                self.addSongNumber(boldtext)
                return u''
            text = self.uncapText(text)
            if self.title == u'':
                self.addTitle(text)
            return text
        if text.strip().startswith(u'('):
            return text
        self.italics = (textportion.CharPosture == ITALIC)
        return text

    def newSong(self):
        """
        A change of song. Store the old, create a new
        ... but only if the last song was complete. If not, stick with it
        """
        if self.song:
            self.finishVerse()
            if not self.checkComplete():
                return
            self.finish()
        self.song = True
        self.setDefaults()
        self.skipToCloseBracket = False
        self.isChorus = False
        self.italics = False
        self.currentVerse = u''

    def addSongNumber(self, song_no):
        """
        Add a song number, store as alternate title. Also use the song
        number to work out which songbook we're in
        """
        self.songNumber = song_no
        self.alternateTitle = song_no + u'.'
        self.songBook_pub = u'Kingsway Publications'
        if int(song_no) <= 640:
            self.songBook = u'Songs of Fellowship 1'
        elif int(song_no) <= 1150:
            self.songBook = u'Songs of Fellowship 2'
        elif int(song_no) <= 1690:
            self.songBook = u'Songs of Fellowship 3'
        elif int(song_no) <= 2200:
            self.songBook = u'Songs of Fellowship 4'
        elif int(song_no) <= 2710:
            self.songBook = u'Songs of Fellowship 5'
        else:
            self.songBook = u'Songs of Fellowship Other'

    def addTitle(self, text):
        """
        Add the title to the song. Strip some leading/trailing punctuation that
        we don't want in a title
        """
        title = text.strip()
        if title.startswith(u'\''):
            title = title[1:]
        if title.endswith(u','):
            title = title[:-1]
        self.title = title
        self.importWizard.incrementProgressBar(u'Processing song ' + title, 0)

    def addSofAuthor(self, text):
        """
        Add the author. OpenLP stores them individually so split by 'and', '&'
        and comma.
        However need to check for "Mr and Mrs Smith" and turn it to
        "Mr Smith" and "Mrs Smith".
        """
        text = text.replace(u' and ', u' & ')
        self.parseAuthor(text)

    def addVerseLine(self, text):
        """
        Add a line to the current verse. If the formatting has changed and
        we're beyond the second line of first verse, then this indicates
        a change of verse. Italics are a chorus
        """
        if self.italics != self.isChorus and ((len(self.verses) > 0) or
            (self.currentVerse.count(u'\n') > 1)):
            self.finishVerse()
        if self.italics:
            self.isChorus = True
        self.currentVerse += text + u'\n'

    def finishVerse(self):
        """
        Verse is finished, store it. Note in book 1+2, some songs are formatted
        incorrectly. Here we try and split songs with missing line breaks into
        the correct number of verses.
        """
        if self.currentVerse.strip() == u'':
            return
        if self.isChorus:
            versetag = u'C'
            splitat = None
        else:
            versetag = u'V'
            splitat = self.verseSplits(self.songNumber)
        if splitat:
            ln = 0
            verse = u''
            for line in self.currentVerse.split(u'\n'):
                ln += 1
                if line == u'' or ln > splitat:
                    self.addSofVerse(verse, versetag)
                    ln = 0
                    if line:
                        verse = line + u'\n'
                    else:
                        verse = u''
                else:
                    verse += line + u'\n'
            if verse:
                self.addSofVerse(verse, versetag)
        else:
            self.addSofVerse(self.currentVerse, versetag)
        self.currentVerse = u''
        self.isChorus = False

    def addSofVerse(self, lyrics, tag):
        self.addVerse(lyrics, tag)
        if not self.isChorus and u'C1' in self.verseOrderListGenerated:
            self.verseOrderListGenerated.append(u'C1')
            self.verseOrderListGenerated_useful = True

    def uncapText(self, text):
        """
        Words in the title are in all capitals, so we lowercase them.
        However some of these words, e.g. referring to God need a leading
        capital letter.

        There is a complicated word "One", which is sometimes lower and
        sometimes upper depending on context. Never mind, keep it lower.
        """
        textarr = re.split(u'(\W+)', text)
        textarr[0] = textarr[0].capitalize()
        for i in range(1, len(textarr)):
            # Do not translate these. Fixed strings in SOF song file
            if textarr[i] in (u'JESUS', u'CHRIST', u'KING', u'ALMIGHTY',
                u'REDEEMER', u'SHEPHERD', u'SON', u'GOD', u'LORD', u'FATHER',
                u'HOLY', u'SPIRIT', u'LAMB', u'YOU', u'YOUR', u'I', u'I\'VE',
                u'I\'M', u'I\'LL', u'SAVIOUR', u'O', u'YOU\'RE', u'HE', u'HIS',
                u'HIM', u'ZION', u'EMMANUEL', u'MAJESTY', u'JESUS\'', u'JIREH',
                u'JUDAH', u'LION', u'LORD\'S', u'ABRAHAM', u'GOD\'S',
                u'FATHER\'S', u'ELIJAH' u'MARTHA', u'CHRISTMAS', u'ALPHA',
                u'OMEGA'):
                textarr[i] = textarr[i].capitalize()
            else:
                textarr[i] = textarr[i].lower()
        text = u''.join(textarr)
        return text

    def verseSplits(self, song_number):
        """
        Because someone at Kingsway forgot to check the 1+2 RTF file,
        some verses were not formatted correctly.
        """
        if song_number == 11:
            return 8
        if song_number == 18:
            return 5
        if song_number == 21:
            return 6
        if song_number == 23:
            return 4
        if song_number == 24:
            return 7
        if song_number == 27:
            return 4
        if song_number == 31:
            return 6
        if song_number == 49:
            return 4
        if song_number == 50:
            return 8
        if song_number == 70:
            return 4
        if song_number == 75:
            return 8
        if song_number == 79:
            return 6
        if song_number == 97:
            return 7
        if song_number == 107:
            return 4
        if song_number == 109:
            return 4
        if song_number == 133:
            return 4
        if song_number == 155:
            return 10
        if song_number == 156:
            return 8
        if song_number == 171:
            return 4
        if song_number == 188:
            return 7
        if song_number == 192:
            return 4
        if song_number == 208:
            return 8
        if song_number == 215:
            return 8
        if song_number == 220:
            return 4
        if song_number == 247:
            return 6
        if song_number == 248:
            return 6
        if song_number == 251:
            return 8
        if song_number == 295:
            return 8
        if song_number == 307:
            return 5
        if song_number == 314:
            return 6
        if song_number == 325:
            return 8
        if song_number == 386:
            return 6
        if song_number == 415:
            return 4
        if song_number == 426:
            return 4
        if song_number == 434:
            return 5
        if song_number == 437:
            return 4
        if song_number == 438:
            return 6
        if song_number == 456:
            return 8
        if song_number == 461:
            return 4
        if song_number == 469:
            return 4
        if song_number == 470:
            return 5
        if song_number == 476:
            return 6
        if song_number == 477:
            return 7
        if song_number == 480:
            return 8
        if song_number == 482:
            return 4
        if song_number == 512:
            return 4
        if song_number == 513:
            return 8
        if song_number == 518:
            return 5
        if song_number == 520:
            return 4
        if song_number == 523:
            return 6
        if song_number == 526:
            return 8
        if song_number == 527:
            return 4
        if song_number == 529:
            return 4
        if song_number == 537:
            return 4
        if song_number == 555:
            return 6
        if song_number == 581:
            return 4
        if song_number == 589:
            return 6
        if song_number == 590:
            return 4
        if song_number == 593:
            return 8
        if song_number == 596:
            return 4
        if song_number == 610:
            return 6
        if song_number == 611:
            return 6
        if song_number == 619:
            return 8
        if song_number == 645:
            return 5
        if song_number == 653:
            return 6
        if song_number == 683:
            return 7
        if song_number == 686:
            return 4
        if song_number == 697:
            return 8
        if song_number == 698:
            return 4
        if song_number == 704:
            return 6
        if song_number == 716:
            return 4
        if song_number == 717:
            return 6
        if song_number == 730:
            return 4
        if song_number == 731:
            return 8
        if song_number == 732:
            return 8
        if song_number == 738:
            return 4
        if song_number == 756:
            return 9
        if song_number == 815:
            return 6
        if song_number == 830:
            return 8
        if song_number == 831:
            return 4
        if song_number == 876:
            return 6
        if song_number == 877:
            return 6
        if song_number == 892:
            return 4
        if song_number == 894:
            return 6
        if song_number == 902:
            return 8
        if song_number == 905:
            return 8
        if song_number == 921:
            return 6
        if song_number == 940:
            return 7
        if song_number == 955:
            return 9
        if song_number == 968:
            return 8
        if song_number == 972:
            return 7
        if song_number == 974:
            return 4
        if song_number == 988:
            return 6
        if song_number == 991:
            return 5
        if song_number == 1002:
            return 8
        if song_number == 1024:
            return 8
        if song_number == 1044:
            return 9
        if song_number == 1088:
            return 6
        if song_number == 1117:
            return 6
        if song_number == 1119:
            return 7
        return None
